# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import timeutils
import requests_mock

from barbicanclient import base
from barbicanclient.tests import test_client
from barbicanclient.v1 import acls


class ACLTestCase(test_client.BaseEntityResource):

    def setUp(self):
        self._setUp('acl', entity_id='d9f95d61-8863-49d3-a045-5c2cb77130b5')

        self.secret_uuid = '8a3108ec-88fc-4f5c-86eb-f37b8ae8358e'
        self.secret_ref = (self.endpoint + '/v1/secrets/' + self.secret_uuid)
        self.secret_acl_ref = '{0}/acl'.format(self.secret_ref)

        self.container_uuid = '83c302c7-86fe-4f07-a277-c4962f121f19'
        self.container_ref = (self.endpoint + '/v1/containers/' +
                              self.container_uuid)
        self.container_acl_ref = '{0}/acl'.format(self.container_ref)

        self.manager = self.client.acls
        self.users1 = ['2d0ee7c681cc4549b6d76769c320d91f',
                       '721e27b8505b499e8ab3b38154705b9e']
        self.users2 = ['2d0ee7c681cc4549b6d76769c320d91f']
        self.created = str(timeutils.utcnow())

    def get_acl_response_data(self, operation_type='read',
                              users=None,
                              project_access=False):
        if users is None:
            users = self.users1
        op_data = {'users': users}
        op_data['project-access'] = project_access
        op_data['created'] = self.created
        op_data['updated'] = str(timeutils.utcnow())
        acl_data = {operation_type: op_data}
        return acl_data


class WhenTestingACLManager(ACLTestCase):

    def test_should_get_secret_acl(self, entity_ref=None, ref_type=None):
        entity_ref = entity_ref or self.secret_ref
        self.responses.get(self.secret_acl_ref,
                           json=self.get_acl_response_data())

        api_resp = self.manager.get(entity_ref=entity_ref, ref_type=ref_type)
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)
        self.assertFalse(api_resp.get('read').project_access)
        self.assertEqual('read', api_resp.get('read').operation_type)
        self.assertIn(api_resp.get('read').acl_ref_relative,
                      self.secret_acl_ref)

    def test_should_get_secret_acl_using_stripped_uuid(self):
        bad_href = "http://badsite.com/secrets/" + self.secret_uuid
        self.test_should_get_secret_acl(bad_href)

    def test_should_get_secret_acl_using_uuid(self):
        self.test_should_get_secret_acl(self.secret_uuid, ref_type='secret')

    def test_should_get_secret_acl_with_extra_trailing_slashes(self):
        self.responses.get(requests_mock.ANY,
                           json=self.get_acl_response_data())
        # check if trailing slashes are corrected in get call.
        self.manager.get(entity_ref=self.secret_ref + '///')
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)

    def test_should_get_container_acl(self, entity_ref=None, ref_type=None):
        entity_ref = entity_ref or self.container_ref
        self.responses.get(self.container_acl_ref,
                           json=self.get_acl_response_data())

        api_resp = self.manager.get(entity_ref=entity_ref, ref_type=ref_type)
        self.assertEqual(self.container_acl_ref,
                         self.responses.last_request.url)
        self.assertFalse(api_resp.get('read').project_access)
        self.assertEqual('read', api_resp.get('read').operation_type)
        self.assertIn(api_resp.get('read').acl_ref_relative,
                      self.container_acl_ref)

    def test_should_get_container_acl_using_stripped_uuid(self):
        bad_href = "http://badsite.com/containers/" + self.container_uuid
        self.test_should_get_container_acl(bad_href)

    def test_should_get_container_acl_using_uuid(self):
        self.test_should_get_container_acl(self.container_uuid,
                                           ref_type='container')

    def test_should_get_container_acl_with_trailing_slashes(self):
        self.responses.get(requests_mock.ANY,
                           json=self.get_acl_response_data())
        # check if trailing slashes are corrected in get call.
        self.manager.get(entity_ref=self.container_ref + '///')
        self.assertEqual(self.container_acl_ref,
                         self.responses.last_request.url)

    def test_should_fail_get_no_href(self):
        self.assertRaises(ValueError, self.manager.get, None)

    def test_should_fail_get_invalid_uri(self):
        # secret_acl URI expected and not secret URI
        self.assertRaises(ValueError, self.manager.get, self.secret_acl_ref)

        self.assertRaises(ValueError, self.manager.get,
                          self.endpoint + '/containers/consumers')
        self.assertRaises(ValueError, self.manager.get,
                          self.endpoint + '/v1/badsecrets/' + self.secret_uuid)
        self.assertRaises(ValueError, self.manager.get,
                          self.endpoint + '/v1/secretsbad/' + self.secret_uuid)
        self.assertRaises(
            ValueError, self.manager.get,
            self.endpoint + '/v1/badcontainers/' + self.container_uuid)
        self.assertRaises(
            ValueError, self.manager.get,
            self.endpoint + '/v1/containersbad/' + self.container_uuid)

    def test_should_fail_get_invalid_ref_type(self):
        self.assertRaises(ValueError, self.manager.get, self.secret_uuid,
                          'foo')
        self.assertRaises(ValueError, self.manager.get, self.secret_uuid,
                          'secrets')
        self.assertRaises(ValueError, self.manager.get, self.container_uuid,
                          'foo')
        self.assertRaises(ValueError, self.manager.get, self.container_uuid,
                          'containers')

    def test_should_create_secret_acl(self):
        entity = self.manager.create(entity_ref=self.secret_ref + '///',
                                     users=self.users1, project_access=True)
        self.assertIsInstance(entity, acls.SecretACL)

        read_acl = entity.read
        # entity ref is kept same as provided input.
        self.assertEqual(self.secret_ref + '///', read_acl.entity_ref)
        self.assertTrue(read_acl.project_access)
        self.assertEqual(self.users1, read_acl.users)
        self.assertEqual(acls.DEFAULT_OPERATION_TYPE, read_acl.operation_type)
        # acl ref removes extra trailing slashes if there
        self.assertEqual(self.secret_acl_ref, read_acl.acl_ref)
        self.assertIsNone(read_acl.created)
        self.assertIsNone(read_acl.updated)

        read_acl_via_get = entity.get('read')
        self.assertEqual(read_acl, read_acl_via_get)

    def test_should_create_secret_acl_for_uuid_ref(self):
        entity = self.manager.create(
            entity_ref=base.calculate_uuid_ref(self.secret_uuid, 'secrets'),
            users=self.users1, project_access=True)
        self.assertIsInstance(entity, acls.SecretACL)

        read_acl = entity.read
        self.assertEqual('secrets/' + self.secret_uuid,
                         read_acl.entity_ref)
        self.assertEqual('secrets/' + self.secret_uuid + '/acl',
                         read_acl.acl_ref)

    def test_should_create_acl_with_users(self, entity_ref=None):
        entity_ref = entity_ref or self.container_ref
        entity = self.manager.create(entity_ref=entity_ref + '///',
                                     users=self.users2, project_access=False)
        self.assertIsInstance(entity, acls.ContainerACL)
        # entity ref is kept same as provided input.
        self.assertEqual(entity_ref + '///', entity.entity_ref)

        read_acl = entity.read
        self.assertFalse(read_acl.project_access)
        self.assertEqual(self.users2, read_acl.users)
        self.assertEqual(acls.DEFAULT_OPERATION_TYPE, read_acl.operation_type)
        # acl ref removes extra trailing slashes if there
        self.assertIn(entity_ref, read_acl.acl_ref,
                      'ACL ref has additional /acl')
        self.assertIn(read_acl.acl_ref_relative, self.container_acl_ref)

    def test_should_create_acl_with_users_stripped_uuid(self):
        bad_href = "http://badsite.com/containers/" + self.container_uuid
        self.test_should_create_acl_with_users(bad_href)

    def test_should_create_acl_with_no_users(self):
        entity = self.manager.create(entity_ref=self.container_ref, users=[])
        read_acl = entity.read
        self.assertEqual([], read_acl.users)
        self.assertEqual(acls.DEFAULT_OPERATION_TYPE, read_acl.operation_type)
        self.assertIsNone(read_acl.project_access)

        read_acl_via_get = entity.get('read')
        self.assertEqual(read_acl, read_acl_via_get)

    def test_create_no_acl_settings(self):

        entity = self.manager.create(entity_ref=self.container_ref)
        self.assertEqual([], entity.operation_acls)
        self.assertEqual(self.container_ref, entity.entity_ref)
        self.assertEqual(self.container_ref + '/acl', entity.acl_ref)

    def test_should_fail_create_invalid_uri(self):

        self.assertRaises(ValueError, self.manager.create,
                          self.endpoint + '/orders')
        self.assertRaises(ValueError, self.manager.create, None)


class WhenTestingACLEntity(ACLTestCase):

    def test_should_submit_acl_with_users_project_access_set(self, href=None):
        href = href or self.secret_ref
        data = {'acl_ref': self.secret_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.secret_acl_ref, json=data)

        entity = self.manager.create(entity_ref=href + '///',
                                     users=self.users1, project_access=True)
        api_resp = entity.submit()
        self.assertEqual(self.secret_acl_ref, api_resp)
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)

    def test_should_submit_acl_with_users_project_access_stripped_uuid(self):
        bad_href = "http://badsite.com/secrets/" + self.secret_uuid
        self.test_should_submit_acl_with_users_project_access_set(bad_href)

    def test_should_submit_acl_with_project_access_set_but_no_users(self):
        data = {'acl_ref': self.secret_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.secret_acl_ref, json=data)

        entity = self.manager.create(entity_ref=self.secret_ref,
                                     project_access=False)
        api_resp = entity.submit()
        self.assertEqual(self.secret_acl_ref, api_resp)
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)
        self.assertFalse(entity.read.project_access)
        self.assertEqual([], entity.get('read').users)

    def test_should_submit_acl_with_user_set_but_not_project_access(self):
        data = {'acl_ref': self.container_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.container_acl_ref, json=data)

        entity = self.manager.create(entity_ref=self.container_ref,
                                     users=self.users2)
        api_resp = entity.submit()
        self.assertEqual(self.container_acl_ref, api_resp)
        self.assertEqual(self.container_acl_ref,
                         self.responses.last_request.url)
        self.assertEqual(self.users2, entity.read.users)
        self.assertIsNone(entity.get('read').project_access)

    def test_should_fail_submit_acl_invalid_secret_uri(self):
        data = {'acl_ref': self.secret_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.secret_acl_ref, json=data)
        entity = self.manager.create(entity_ref=self.secret_acl_ref + '///',
                                     users=self.users1, project_access=True)
        # Submit checks provided URI is entity URI and not ACL URI.
        self.assertRaises(ValueError, entity.submit)

        entity = self.manager.create(entity_ref=self.secret_ref,
                                     users=self.users1, project_access=True)
        entity._entity_ref = None
        self.assertRaises(ValueError, entity.submit)

        entity = self.manager.create(entity_ref=self.secret_ref,
                                     users=self.users1, project_access=True)
        entity._entity_ref = self.container_ref  # expected secret uri here
        self.assertRaises(ValueError, entity.submit)

    def test_should_fail_submit_acl_invalid_container_uri(self):
        """Adding tests for container URI validation.

        Container URI validation is different from secret URI validation.
        That's why adding separate tests for code coverage.
        """

        data = {'acl_ref': self.container_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.container_acl_ref, json=data)
        entity = self.manager.create(entity_ref=self.container_acl_ref + '///',
                                     users=self.users1, project_access=True)
        # Submit checks provided URI is entity URI and not ACL URI.
        self.assertRaises(ValueError, entity.submit)

        entity = self.manager.create(entity_ref=self.container_ref,
                                     users=self.users1, project_access=True)
        entity._entity_ref = None
        self.assertRaises(ValueError, entity.submit)

        entity = self.manager.create(entity_ref=self.container_ref,
                                     users=self.users1, project_access=True)
        entity._entity_ref = self.secret_ref  # expected container uri here
        self.assertRaises(ValueError, entity.submit)

    def test_should_fail_submit_acl_no_acl_data(self):
        data = {'acl_ref': self.secret_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.secret_acl_ref, json=data)
        entity = self.manager.create(entity_ref=self.secret_ref + '///')
        # Submit checks that ACL setting data is there or not.
        self.assertRaises(ValueError, entity.submit)

    def test_should_fail_submit_acl_input_users_as_not_list(self):
        data = {'acl_ref': self.secret_acl_ref}
        # register put acl URI with expected acl ref in response
        self.responses.put(self.secret_acl_ref, json=data)
        entity = self.manager.create(entity_ref=self.secret_ref,
                                     users='u1')
        # Submit checks that input users are provided as list or not
        self.assertRaises(ValueError, entity.submit)

    def test_should_load_acls_data(self):
        self.responses.get(
            self.container_acl_ref, json=self.get_acl_response_data(
                users=self.users2, project_access=True))

        entity = self.manager.create(entity_ref=self.container_ref,
                                     users=self.users1)
        self.assertEqual(self.container_ref, entity.entity_ref)
        self.assertEqual(self.container_acl_ref, entity.acl_ref)

        entity.load_acls_data()

        self.assertEqual(self.users2, entity.read.users)
        self.assertTrue(entity.get('read').project_access)
        self.assertEqual(timeutils.parse_isotime(self.created),
                         entity.read.created)
        self.assertEqual(timeutils.parse_isotime(self.created),
                         entity.get('read').created)

        self.assertEqual(1, len(entity.operation_acls))
        self.assertEqual(self.container_acl_ref, entity.get('read').acl_ref)
        self.assertEqual(self.container_ref, entity.read.entity_ref)

    def test_should_add_operation_acl(self):
        entity = self.manager.create(entity_ref=self.secret_ref + '///',
                                     users=self.users1, project_access=True)
        self.assertIsInstance(entity, acls.SecretACL)

        entity.add_operation_acl(users=self.users2, project_access=False,
                                 operation_type='read')

        read_acl = entity.read
        # entity ref is kept same as provided input.
        self.assertEqual(self.secret_ref + '/acl', read_acl.acl_ref)
        self.assertFalse(read_acl.project_access)
        self.assertEqual(self.users2, read_acl.users)
        self.assertEqual(acls.DEFAULT_OPERATION_TYPE, read_acl.operation_type)

        entity.add_operation_acl(users=[], project_access=False,
                                 operation_type='dummy')
        dummy_acl = entity.get('dummy')
        self.assertFalse(dummy_acl.project_access)
        self.assertEqual([], dummy_acl.users)

    def test_acl_entity_properties(self):

        entity = self.manager.create(entity_ref=self.secret_ref,
                                     users=self.users2)
        self.assertEqual(self.secret_ref, entity.entity_ref)
        self.assertEqual(self.secret_acl_ref, entity.acl_ref)

        self.assertEqual(self.users2, entity.read.users)
        self.assertEqual(self.users2, entity.get('read').users)
        self.assertIsNone(entity.read.project_access)
        self.assertIsNone(entity.get('read').project_access)
        self.assertIsNone(entity.read.created)
        self.assertIsNone(entity.get('read').created)
        self.assertEqual('read', entity.read.operation_type)
        self.assertEqual('read', entity.get('read').operation_type)

        self.assertEqual(1, len(entity.operation_acls))
        self.assertEqual(self.secret_acl_ref, entity.read.acl_ref)
        self.assertEqual(self.secret_acl_ref, entity.get('read').acl_ref)
        self.assertEqual(self.secret_ref, entity.read.entity_ref)

        self.assertIsNone(entity.get('dummyOperation'))

        entity.read.users = ['u1']
        entity.read.project_access = False
        entity.read.operation_type = 'my_operation'
        self.assertFalse(entity.get('my_operation').project_access)
        self.assertEqual(['u1'], entity.get('my_operation').users)

        self.assertRaises(AttributeError, lambda x: x.dummy_operation, entity)

    def test_get_formatted_data(self):

        s_entity = acls.SecretACL(api=None,
                                  entity_ref=self.secret_ref,
                                  users=self.users1)

        data = s_entity.read._get_formatted_data()

        self.assertEqual(acls.DEFAULT_OPERATION_TYPE, data[0])
        self.assertIsNone(data[1])
        self.assertEqual(self.users1, data[2])
        self.assertIsNone(data[3])  # created
        self.assertIsNone(data[4])  # updated
        self.assertEqual(self.secret_acl_ref, data[5])

        c_entity = acls.ContainerACL(api=None,
                                     entity_ref=self.container_ref,
                                     users=self.users2, created=self.created)

        data = c_entity.get('read')._get_formatted_data()

        self.assertEqual(acls.DEFAULT_OPERATION_TYPE, data[0])
        self.assertIsNone(data[1])
        self.assertEqual(self.users2, data[2])
        self.assertEqual(timeutils.parse_isotime(self.created).isoformat(),
                         data[3])  # created
        self.assertIsNone(data[4])  # updated
        self.assertEqual(self.container_acl_ref, data[5])

    def test_should_secret_acl_remove(self, entity_ref=None):
        entity_ref = entity_ref or self.secret_ref
        self.responses.delete(self.secret_acl_ref)

        entity = self.manager.create(entity_ref=entity_ref,
                                     users=self.users2)

        api_resp = entity.remove()
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)
        self.assertIsNone(api_resp)

    def test_should_secret_acl_remove_uri_with_slashes(self):
        self.responses.delete(self.secret_acl_ref)

        # check if trailing slashes are corrected in delete call.
        entity = self.manager.create(entity_ref=self.secret_ref + '///',
                                     users=self.users2)
        entity.remove()
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)

        self.responses.delete(self.container_acl_ref)

    def test_should_secret_acl_remove_stripped_uuid(self):
        bad_href = "http://badsite.com/secrets/" + self.secret_uuid
        self.test_should_secret_acl_remove(bad_href)

    def test_should_container_acl_remove(self, entity_ref=None):
        entity_ref = entity_ref or self.container_ref
        self.responses.delete(self.container_acl_ref)

        entity = self.manager.create(entity_ref=entity_ref)
        entity.remove()
        self.assertEqual(self.container_acl_ref,
                         self.responses.last_request.url)

    def test_should_container_acl_remove_stripped_uuid(self):
        bad_href = "http://badsite.com/containers/" + self.container_uuid
        self.test_should_container_acl_remove(bad_href)

    def test_should_fail_acl_remove_invalid_uri(self):
        # secret_acl URI expected and not secret acl URI
        entity = self.manager.create(entity_ref=self.secret_acl_ref)
        self.assertRaises(ValueError, entity.remove)

        entity = self.manager.create(entity_ref=self.container_acl_ref)
        self.assertRaises(ValueError, entity.remove)

        entity = self.manager.create(entity_ref=self.container_ref +
                                     '/consumers')
        self.assertRaises(ValueError, entity.remove)

        # check to make sure UUID is passed in
        entity = self.manager.create(entity_ref=self.endpoint + '/secrets' +
                                     '/consumers')
        self.assertRaises(ValueError, entity.remove)

    def test_should_per_operation_acl_remove(self):
        self.responses.get(self.secret_acl_ref,
                           json=self.get_acl_response_data(users=self.users2,
                                                           project_access=True)
                           )
        self.responses.delete(self.secret_acl_ref)

        entity = self.manager.create(entity_ref=self.secret_ref,
                                     users=self.users2)

        api_resp = entity.read.remove()
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)
        self.assertIsNone(api_resp)
        self.assertEqual(0, len(entity.operation_acls))

        # now try case where there are 2 operation acls defined
        # and one operation specific acl is removed. In that case,
        # entity.submit() is called instead of remove to update rest of entity

        acl_data = self.get_acl_response_data(users=self.users2,
                                              project_access=True)

        data = self.get_acl_response_data(users=self.users1,
                                          operation_type='write',
                                          project_access=False)
        acl_data['write'] = data['write']

        self.responses.get(self.secret_acl_ref, json=acl_data)
        self.responses.put(self.secret_acl_ref, json={})
        # check if trailing slashes are corrected in delete call.
        entity = self.manager.create(entity_ref=self.secret_ref,
                                     users=self.users2)
        entity.read.remove()
        self.assertEqual(self.secret_acl_ref,
                         self.responses.last_request.url)
        self.assertEqual(1, len(entity.operation_acls))
        self.assertEqual('write', entity.operation_acls[0].operation_type)
        self.assertEqual(self.users1, entity.operation_acls[0].users)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from functionaltests.cli.v1.behaviors import base_behaviors


class SecretBehaviors(base_behaviors.BaseBehaviors):

    def __init__(self):
        super(SecretBehaviors, self).__init__()
        self.LOG = logging.getLogger(type(self).__name__)
        self.secret_hrefs_to_delete = []

    def update_secret(self,
                      secret_href,
                      payload):
        """Update a secret

        :param secret_href the href to the secret to update.
        :param payload the payload to put into the secret.
        :param payload_content_type the payload content type.
        """
        argv = ['secret', 'update']
        self.add_auth_and_endpoint(argv)
        argv.extend([secret_href])
        argv.extend([payload])

        stdout, stderr = self.issue_barbican_command(argv)

    def delete_secret(self, secret_href):
        """Delete a secret

        :param secret_href the href to the secret to delete
        """
        argv = ['secret', 'delete', '--force']
        self.add_auth_and_endpoint(argv)
        argv.extend([secret_href])

        stdout, stderr = self.issue_barbican_command(argv)

        self.secret_hrefs_to_delete.remove(secret_href)

    def store_secret(self, payload=b"Payload for testing", store_argv=[]):
        """Store (aka create) a secret

        The store_argv parameter allows additional command line parameters for
        the store operation to be specified. This can be used to specify -a for
        algorithm as an example.

        :param payload The payload to use when storing the secret
        :param store_argv The store command line parameters

        :return: the href to the newly created secret
        """
        argv = ['secret', 'store']
        self.add_auth_and_endpoint(argv)

        if payload is not None and not isinstance(payload, str):
            # Payload shouldn't be bytes but string on Python3
            payload = payload.decode('utf-8')

        argv.extend(['--payload', payload])
        argv.extend(store_argv)

        stdout, stderr = self.issue_barbican_command(argv)

        secret_data = self._prettytable_to_dict(stdout)

        secret_href = secret_data['Secret href']
        self.secret_hrefs_to_delete.append(secret_href)
        return secret_href

    def store_secret_file(self, filename="/tmp/storesecret", store_argv=[]):
        """Store (aka create) a secret from file

        The store_argv parameter allows additional command line parameters for
        the store operation to be specified. This can be used to specify -a for
        algorithm as an example.

        :param payload The payload to use when storing the secret
        :param store_argv The store command line parameters

        :return: the href to the newly created secret
        """
        argv = ['secret', 'store']
        self.add_auth_and_endpoint(argv)
        argv.extend(['--file', filename])
        argv.extend(store_argv)

        stdout, stderr = self.issue_barbican_command(argv)

        secret_data = self._prettytable_to_dict(stdout)

        secret_href = secret_data['Secret href']
        self.secret_hrefs_to_delete.append(secret_href)
        return secret_href

    def get_secret(self, secret_href):
        """Get a secret

        :param: the href to a secret
        :return: dict of secret values, or an empty dict if the secret
        is not found.
        """
        argv = ['secret', 'get']
        self.add_auth_and_endpoint(argv)
        argv.extend([secret_href])

        stdout, stderr = self.issue_barbican_command(argv)

        if '4xx Client error: Not Found' in stderr:
            return {}

        secret_data = self._prettytable_to_dict(stdout)
        return secret_data

    def get_secret_payload(self, secret_href, raw=False):
        """Get a secret

        :param: the href to a secret
        :param raw if True then add "-f value" to get raw payload (ie not
        within a PrettyTable).  If False then omit -f.
        :return: string representing the secret payload.
        """
        argv = ['secret', 'get']
        self.add_auth_and_endpoint(argv)
        argv.extend([secret_href])
        argv.extend(['--payload'])
        if raw:
            argv.extend(['-f', 'value'])

        stdout, stderr = self.issue_barbican_command(argv)

        if '4xx Client error: Not Found' in stderr:
            return {}

        if raw:
            secret = stdout.rstrip()
        else:
            secret_data = self._prettytable_to_dict(stdout)
            secret = secret_data['Payload']

        return secret

    def get_secret_file(self, secret_href, filename='/tmp/getsecret'):
        """Get a secret and store in a file

        :param: the href to a secret
        :param filename: name of file to store secret in
        :return string representing the file name.
        """
        argv = ['secret', 'get']
        self.add_auth_and_endpoint(argv)
        argv.extend([secret_href])
        argv.extend(['--file', filename])

        stdout, stderr = self.issue_barbican_command(argv)

        if '4xx Client error: Not Found' in stderr:
            return {}

        return filename

    def list_secrets(self):
        """List secrets

        :return: a list of secrets
        """
        argv = ['secret', 'list']

        self.add_auth_and_endpoint(argv)
        stdout, stderr = self.issue_barbican_command(argv)

        secret_list = self._prettytable_to_list(stdout)
        return secret_list

    def delete_all_created_secrets(self):
        """Delete all secrets that we created"""
        # Create a copy of the list -- otherwise delete_secret will remove
        # items from the list as we are iterating over it
        secrets_to_delete = list(self.secret_hrefs_to_delete)
        for href in secrets_to_delete:
            self.delete_secret(href)

    def read_secret_test_file(self, filename='/tmp/getsecret'):
        """Read payload from file used in testing

        :param filename: name of file to write
        :return contents of the file
        """
        with open(filename, "rb") as myfile:
            data = myfile.read()
        return data.decode('utf-8')

    def write_secret_test_file(self, filename='/tmp/storesecret',
                               payload="Payload for testing"):
        """Write payload to file for use in testing

        :param filename: name of file to write
        :param payload: data to store
        :return
        """
        myfile = open(filename, "wb")
        myfile.write(payload.encode('utf-8'))
        myfile.close()
        return



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functionaltests.cli.base import CmdLineTestCase
from functionaltests.cli.v1.behaviors.secret_behaviors import SecretBehaviors
from functionaltests.common import keys
from functionaltests import utils
from testtools import testcase


@utils.parameterized_test_case
class SecretTestCase(CmdLineTestCase):

    def setUp(self):
        super(SecretTestCase, self).setUp()
        self.secret_behaviors = SecretBehaviors()
        self.expected_payload = "Top secret payload for secret smoke tests"
        self.payload_content_type = "text/plain"

    def tearDown(self):
        super(SecretTestCase, self).tearDown()
        self.secret_behaviors.delete_all_created_secrets()

    @utils.parameterized_dataset({
        'symmetric': ['symmetric',
                      'aes',
                      '128',
                      (b'\x00\x01\x02\x03\x04\x05\x06\x07'
                       b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'private': ['private',
                    'rsa',
                    '2048',
                    keys.get_private_key_pem()],
        'public': ['public',
                   'rsa',
                   '2048',
                   keys.get_public_key_pem()],
        'certificate': ['certificate',
                        'rsa',
                        '2048',
                        keys.get_certificate_pem()],
        'opaque': ['opaque',
                   None,
                   None,
                   (b'\x00\x01\x02\x03\x04\x05\x06\x07')],
        'passphrase': ['passphrase',
                       None,
                       None,
                       keys.get_passphrase_txt()],
    })
    @testcase.attr('positive')
    def test_secret_store_with_secret_type(self, secret_type, algorithm,
                                           bit_length, secret):
        payload = secret
        secret_argv = ['--secret-type', secret_type]
        if algorithm:
            secret_argv.extend(['--algorithm', algorithm])
        if bit_length:
            secret_argv.extend(['--bit-length', bit_length])
        secret_href = self.secret_behaviors.store_secret(payload, secret_argv)
        self.assertIsNotNone(secret_href)

        secret = self.secret_behaviors.get_secret(secret_href)
        self.assertEqual(secret_href, secret['Secret href'])

    @testcase.attr('positive')
    def test_secret_store(self):
        secret_href = self.secret_behaviors.store_secret()
        self.assertIsNotNone(secret_href)

        secret = self.secret_behaviors.get_secret(secret_href)
        self.assertEqual(secret_href, secret['Secret href'])

    @testcase.attr('positive')
    def test_secret_update(self):
        secret_href = self.secret_behaviors.store_secret(
            payload=None)

        payload = 'time for an ice cold!!!'
        self.assertIsNotNone(secret_href)
        self.secret_behaviors.update_secret(secret_href,
                                            payload)

        payload_update = self.secret_behaviors.get_secret_payload(secret_href)
        self.assertEqual(payload, payload_update)

    @testcase.attr('positive')
    def test_secret_list(self):
        secrets_to_create = 10
        for _ in range(secrets_to_create):
            self.secret_behaviors.store_secret()
        secret_list = self.secret_behaviors.list_secrets()
        self.assertGreaterEqual(len(secret_list), secrets_to_create)

    @testcase.attr('positive')
    def test_secret_delete(self):
        secret_href = self.secret_behaviors.store_secret()
        self.secret_behaviors.delete_secret(secret_href)

        secret = self.secret_behaviors.get_secret(secret_href)
        self.assertEqual(0, len(secret))

    @testcase.attr('positive')
    def test_secret_get(self):
        secret_href = self.secret_behaviors.store_secret()
        secret = self.secret_behaviors.get_secret(secret_href)
        self.assertIsNotNone(secret)

    @testcase.attr('positive')
    def test_secret_get_payload(self):
        secret_href = self.secret_behaviors.store_secret(
            payload=self.expected_payload)
        payload = self.secret_behaviors.get_secret_payload(secret_href)
        self.assertEqual(payload, self.expected_payload)

    @testcase.attr('positive')
    def test_secret_get_raw_payload(self):
        secret_href = self.secret_behaviors.store_secret(
            payload=self.expected_payload)
        payload = self.secret_behaviors.get_secret_payload(secret_href,
                                                           raw=True)
        self.assertEqual(payload, self.expected_payload)

    @testcase.attr('positive')
    def test_secret_file_parameter_read(self):
        secret_href = self.secret_behaviors.store_secret(
            payload=self.expected_payload)
        self.secret_behaviors.get_secret_file(secret_href=secret_href)
        payload = self.secret_behaviors.read_secret_test_file()
        self.assertEqual(payload, self.expected_payload)

    @testcase.attr('positive')
    def test_secret_file_parameter_write(self):
        self.secret_behaviors.write_secret_test_file(
            payload=self.expected_payload)
        secret_href = self.secret_behaviors.store_secret_file()
        payload = self.secret_behaviors.get_secret_payload(secret_href)
        self.assertEqual(payload, self.expected_payload)



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Command-line interface to the Barbican API.
"""

from collections import namedtuple
import logging
import sys

from cliff import app
from cliff import command
from cliff import commandmanager
from cliff import complete
from cliff import help
from keystoneauth1 import identity
from keystoneauth1 import loading
from keystoneauth1 import session

import barbicanclient
from barbicanclient._i18n import _LW
from barbicanclient import client


LOG = logging.getLogger(__name__)


_DEFAULT_IDENTITY_API_VERSION = '3'
_IDENTITY_API_VERSION_2 = ['2', '2.0']
_IDENTITY_API_VERSION_3 = ['3']


class Barbican(app.App):
    """Barbican command line interface."""

    # verbose logging levels
    WARNING_LEVEL = 0
    INFO_LEVEL = 1
    DEBUG_LEVEL = 2
    CONSOLE_MESSAGE_FORMAT = '%(message)s'
    DEBUG_MESSAGE_FORMAT = '%(levelname)s: %(name)s %(message)s'

    def __init__(self, **kwargs):
        self.client = None

        # Patch command.Command to add a default auth_required = True
        command.Command.auth_required = True

        # Some commands do not need authentication
        help.HelpCommand.auth_required = False
        complete.CompleteCommand.auth_required = False

        super(Barbican, self).__init__(
            description=__doc__.strip(),
            version=barbicanclient.__version__,
            command_manager=commandmanager.CommandManager(
                'openstack.key_manager.v1'),
            deferred_help=True,
            **kwargs
        )

    def check_auth_arguments(self, args, api_version=None, raise_exc=False):
        """Verifies that we have the correct arguments for authentication

        Supported Keystone v3 combinations:
            - Project Id
            - Project Name + Project Domain Name
            - Project Name + Project Domain Id
        Supported Keystone v2 combinations:
            - Tenant Id
            - Tenant Name
        """
        successful = True
        v3_arg_combinations = [
            args.os_project_id,
            args.os_project_name and args.os_project_domain_name,
            args.os_project_name and args.os_project_domain_id
        ]
        v2_arg_combinations = [args.os_tenant_id, args.os_tenant_name]

        # Keystone V3
        if not api_version or api_version == _DEFAULT_IDENTITY_API_VERSION:
            if not any(v3_arg_combinations):
                msg = ('ERROR: please specify the following --os-project-id or'
                       ' (--os-project-name and --os-project-domain-name) or '
                       ' (--os-project-name and --os-project-domain-id)')
                successful = False
        # Keystone V2
        else:
            if not any(v2_arg_combinations):
                msg = ('ERROR: please specify --os-tenant-id or'
                       ' --os-tenant-name')
                successful = False

        if not successful and raise_exc:
            raise Exception(msg)

        return successful

    def build_kwargs_based_on_version(self, args, api_version=None):
        if not api_version or api_version == _DEFAULT_IDENTITY_API_VERSION:
            kwargs = {
                'project_id': args.os_project_id,
                'project_name': args.os_project_name,
                'user_domain_id': args.os_user_domain_id,
                'user_domain_name': args.os_user_domain_name,
                'project_domain_id': args.os_project_domain_id,
                'project_domain_name': args.os_project_domain_name
            }
        else:
            kwargs = {
                'tenant_name': args.os_tenant_name,
                'tenant_id': args.os_tenant_id
            }

        # Return a dictionary with only the populated (not None) values
        return dict((k, v) for (k, v) in kwargs.items() if v)

    def create_keystone_session(
            self, args, api_version, kwargs_dict, auth_type
    ):
        # Make sure we have the correct arguments to function
        self.check_auth_arguments(args, api_version, raise_exc=True)

        kwargs = self.build_kwargs_based_on_version(args, api_version)
        kwargs.update(kwargs_dict)

        _supported_version = _IDENTITY_API_VERSION_2 + _IDENTITY_API_VERSION_3
        if not api_version or api_version not in _supported_version:
            self.stderr.write(
                "WARNING: The identity version <{0}> is not in supported "
                "versions <{1}>, falling back to <{2}>.".format(
                    api_version,
                    _IDENTITY_API_VERSION_2 + _IDENTITY_API_VERSION_3,
                    _DEFAULT_IDENTITY_API_VERSION
                )
            )
        method = identity.Token if auth_type == 'token' else identity.Password

        auth = method(**kwargs)
        cacert = args.os_cacert
        cert = args.os_cert
        key = args.os_key
        insecure = args.insecure
        if insecure:
            verify = False
        else:
            verify = cacert or True
        if cert and key:
            cert = (cert, key)
        return session.Session(auth=auth, verify=verify, cert=cert)

    def create_client(self, args):
        created_client = None
        endpoint_filter_kwargs = self._get_endpoint_filter_kwargs(args)

        api_version = args.os_identity_api_version
        if args.no_auth and args.os_auth_url:
            raise Exception(
                'ERROR: argument --os-auth-url/-A: not allowed '
                'with argument --no-auth/-N'
            )

        if args.no_auth:
            if not all([args.endpoint, args.os_tenant_id or
                        args.os_project_id]):
                raise Exception(
                    'ERROR: please specify --endpoint and '
                    '--os-project-id (or --os-tenant-id)')
            created_client = client.Client(
                endpoint=args.endpoint,
                project_id=args.os_tenant_id or args.os_project_id,
                verify=not args.insecure,
                **endpoint_filter_kwargs
            )
        # Token-based authentication
        elif args.os_auth_token:
            if not args.os_auth_url:
                raise Exception('ERROR: please specify --os-auth-url')
            token_kwargs = {
                'auth_url': args.os_auth_url,
                'token': args.os_auth_token
            }
            session = self.create_keystone_session(
                args, api_version, token_kwargs, auth_type='token'
            )
            created_client = client.Client(
                session=session,
                endpoint=args.endpoint,
                **endpoint_filter_kwargs
            )

        # Password-based authentication
        elif args.os_auth_url:
            password_kwargs = {
                'auth_url': args.os_auth_url,
                'password': args.os_password,
                'user_id': args.os_user_id,
                'username': args.os_username
            }
            session = self.create_keystone_session(
                args, api_version, password_kwargs, auth_type='password'
            )
            created_client = client.Client(
                session=session,
                endpoint=args.endpoint,
                **endpoint_filter_kwargs
            )
        else:
            raise Exception('ERROR: please specify authentication credentials')

        return created_client

    def _get_endpoint_filter_kwargs(self, args):
        endpoint_filter_keys = ('interface', 'service_type', 'service_name',
                                'barbican_api_version', 'region_name')
        kwargs = dict((key, getattr(args, key)) for key in endpoint_filter_keys
                      if getattr(args, key, None))
        if 'barbican_api_version' in kwargs:
            kwargs['version'] = kwargs.pop('barbican_api_version')
        return kwargs

    def build_option_parser(self, description, version, argparse_kwargs=None):
        """Introduces global arguments for the application.

        This is inherited from the framework.
        """
        parser = super(Barbican, self).build_option_parser(
            description, version, argparse_kwargs)
        parser.add_argument('--no-auth', '-N', action='store_true',
                            help='Do not use authentication.')
        parser.add_argument('--os-identity-api-version',
                            metavar='<identity-api-version>',
                            default=client.env('OS_IDENTITY_API_VERSION'),
                            help='Specify Identity API version to use. '
                            'Defaults to env[OS_IDENTITY_API_VERSION]'
                            ' or 3.')
        parser.add_argument('--os-auth-url', '-A',
                            metavar='<auth-url>',
                            default=client.env('OS_AUTH_URL'),
                            help='Defaults to env[OS_AUTH_URL].')
        parser.add_argument('--os-username', '-U',
                            metavar='<auth-user-name>',
                            default=client.env('OS_USERNAME'),
                            help='Defaults to env[OS_USERNAME].')
        parser.add_argument('--os-user-id',
                            metavar='<auth-user-id>',
                            default=client.env('OS_USER_ID'),
                            help='Defaults to env[OS_USER_ID].')
        parser.add_argument('--os-password', '-P',
                            metavar='<auth-password>',
                            default=client.env('OS_PASSWORD'),
                            help='Defaults to env[OS_PASSWORD].')
        parser.add_argument('--os-user-domain-id',
                            metavar='<auth-user-domain-id>',
                            default=client.env('OS_USER_DOMAIN_ID'),
                            help='Defaults to env[OS_USER_DOMAIN_ID].')
        parser.add_argument('--os-user-domain-name',
                            metavar='<auth-user-domain-name>',
                            default=client.env('OS_USER_DOMAIN_NAME'),
                            help='Defaults to env[OS_USER_DOMAIN_NAME].')
        parser.add_argument('--os-tenant-name', '-T',
                            metavar='<auth-tenant-name>',
                            default=client.env('OS_TENANT_NAME'),
                            help='Defaults to env[OS_TENANT_NAME].')
        parser.add_argument('--os-tenant-id', '-I',
                            metavar='<tenant-id>',
                            default=client.env('OS_TENANT_ID'),
                            help='Defaults to env[OS_TENANT_ID].')
        parser.add_argument('--os-project-id',
                            metavar='<auth-project-id>',
                            default=client.env('OS_PROJECT_ID'),
                            help='Another way to specify tenant ID. '
                                 'This option is mutually exclusive with '
                                 ' --os-tenant-id. '
                            'Defaults to env[OS_PROJECT_ID].')
        parser.add_argument('--os-project-name',
                            metavar='<auth-project-name>',
                            default=client.env('OS_PROJECT_NAME'),
                            help='Another way to specify tenant name. '
                                 'This option is mutually exclusive with '
                                 ' --os-tenant-name. '
                                 'Defaults to env[OS_PROJECT_NAME].')
        parser.add_argument('--os-project-domain-id',
                            metavar='<auth-project-domain-id>',
                            default=client.env('OS_PROJECT_DOMAIN_ID'),
                            help='Defaults to env[OS_PROJECT_DOMAIN_ID].')
        parser.add_argument('--os-project-domain-name',
                            metavar='<auth-project-domain-name>',
                            default=client.env('OS_PROJECT_DOMAIN_NAME'),
                            help='Defaults to env[OS_PROJECT_DOMAIN_NAME].')
        parser.add_argument('--os-auth-token',
                            metavar='<auth-token>',
                            default=client.env('OS_AUTH_TOKEN'),
                            help='Defaults to env[OS_AUTH_TOKEN].')
        parser.add_argument('--endpoint', '-E',
                            metavar='<barbican-url>',
                            default=client.env('BARBICAN_ENDPOINT'),
                            help='Defaults to env[BARBICAN_ENDPOINT].')
        parser.add_argument('--interface',
                            metavar='<barbican-interface>',
                            default=client.env('BARBICAN_INTERFACE'),
                            help='Defaults to env[BARBICAN_INTERFACE].')
        parser.add_argument('--service-type',
                            metavar='<barbican-service-type>',
                            default=client.env('BARBICAN_SERVICE_TYPE'),
                            help='Defaults to env[BARBICAN_SERVICE_TYPE].')
        parser.add_argument('--service-name',
                            metavar='<barbican-service-name>',
                            default=client.env('BARBICAN_SERVICE_NAME'),
                            help='Defaults to env[BARBICAN_SERVICE_NAME].')
        parser.add_argument('--region-name',
                            metavar='<barbican-region-name>',
                            default=client.env('BARBICAN_REGION_NAME'),
                            help='Defaults to env[BARBICAN_REGION_NAME].')
        parser.add_argument('--barbican-api-version',
                            metavar='<barbican-api-version>',
                            default=client.env('BARBICAN_API_VERSION'),
                            help='Defaults to env[BARBICAN_API_VERSION].')
        parser.epilog = ('See "barbican help COMMAND" for help '
                         'on a specific command.')
        loading.register_session_argparse_arguments(parser)
        return parser

    def prepare_to_run_command(self, cmd):
        """Prepares to run the command

        Checks if the minimal parameters are provided and creates the
        client interface.
        This is inherited from the framework.
        """
        self.client_manager = namedtuple('ClientManager', 'key_manager')
        if cmd.auth_required:
            # NOTE(liujiong): cliff sets log level to DEBUG in run function,
            # need to overwrite this configuration to depress DEBUG messages.
            self.configure_logging()
            self.client_manager.key_manager = self.create_client(self.options)

    def run(self, argv):
        # If no arguments are provided, usage is displayed
        if not argv:
            self.stderr.write(self.parser.format_usage())
            return 1
        return super(Barbican, self).run(argv)

    def configure_logging(self):
        """Create logging handlers for any log output."""
        root_logger = logging.getLogger('')
        # Set log level to INFO
        root_logger.setLevel(logging.INFO)

        # Send higher-level messages to the console via stderr
        console = logging.StreamHandler(self.stderr)
        console_level = {self.WARNING_LEVEL: logging.WARNING,
                         self.INFO_LEVEL: logging.INFO,
                         self.DEBUG_LEVEL: logging.DEBUG,
                         }.get(self.options.verbose_level, logging.INFO)
        if logging.DEBUG == console_level:
            formatter = logging.Formatter(self.DEBUG_MESSAGE_FORMAT)
        else:
            formatter = logging.Formatter(self.CONSOLE_MESSAGE_FORMAT)
        console.setFormatter(formatter)
        root_logger.addHandler(console)
        return


def main(argv=sys.argv[1:]):
    logging.basicConfig()
    LOG.warning(_LW("This Barbican CLI interface has been deprecated and "
                    "will be removed in the O release. Please use the "
                    "openstack unified client instead."))
    barbican_app = Barbican()
    return barbican_app.run(argv)


if __name__ == '__main__':   # pragma: no cover
    sys.exit(main(sys.argv[1:]))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from oslo_serialization import jsonutils
from oslo_utils import uuidutils
from requests_mock.contrib import fixture
import testtools

import barbicanclient.barbican

# these are copied from python-keystoneclient tests
BASE_HOST = 'http://keystone.example.com'
BASE_URL = "%s:5000/" % BASE_HOST
UPDATED = '2013-03-06T00:00:00Z'

V2_URL = "%sv2.0" % BASE_URL
V2_DESCRIBED_BY_HTML = {'href': 'http://docs.openstack.org/api/'
                                'openstack-identity-service/2.0/content/',
                        'rel': 'describedby',
                        'type': 'text/html'}
V2_DESCRIBED_BY_PDF = {'href': 'http://docs.openstack.org/api/openstack-ident'
                               'ity-service/2.0/identity-dev-guide-2.0.pdf',
                       'rel': 'describedby',
                       'type': 'application/pdf'}

V2_VERSION = {'id': 'v2.0',
              'links': [{'href': V2_URL, 'rel': 'self'},
                        V2_DESCRIBED_BY_HTML, V2_DESCRIBED_BY_PDF],
              'status': 'stable',
              'updated': UPDATED}

V3_URL = "%sv3" % BASE_URL
V3_MEDIA_TYPES = [{'base': 'application/json',
                   'type': 'application/vnd.openstack.identity-v3+json'},
                  {'base': 'application/xml',
                   'type': 'application/vnd.openstack.identity-v3+xml'}]

V3_VERSION = {'id': 'v3',
              'links': [{'href': V3_URL, 'rel': 'self'}],
              'media-types': V3_MEDIA_TYPES,
              'status': 'stable',
              'updated': UPDATED}


def _create_version_list(versions):
    return jsonutils.dumps({'versions': {'values': versions}})


def _create_single_version(version):
    return jsonutils.dumps({'version': version})


V3_VERSION_LIST = _create_version_list([V3_VERSION, V2_VERSION])
V2_VERSION_LIST = _create_version_list([V2_VERSION])

V3_VERSION_ENTRY = _create_single_version(V3_VERSION)
V2_VERSION_ENTRY = _create_single_version(V2_VERSION)

BARBICAN_ENDPOINT = 'http://www.barbican.com/v1'


def _get_normalized_token_data(**kwargs):
    ref = copy.deepcopy(kwargs)
    # normalized token data
    ref['user_id'] = ref.get('user_id',
                             uuidutils.generate_uuid(dashed=False))
    ref['username'] = ref.get('username',
                              uuidutils.generate_uuid(dashed=False))
    ref['project_id'] = ref.get(
        'project_id',
        ref.get('tenant_id', uuidutils.generate_uuid(dashed=False)))
    ref['project_name'] = ref.get(
        'tenant_name',
        ref.get('tenant_name', uuidutils.generate_uuid(dashed=False)))
    ref['user_domain_id'] = ref.get(
        'user_domain_id',
        uuidutils.generate_uuid(dashed=False))
    ref['user_domain_name'] = ref.get(
        'user_domain_name',
        uuidutils.generate_uuid(dashed=False))
    ref['project_domain_id'] = ref.get(
        'project_domain_id',
        uuidutils.generate_uuid(dashed=False))
    ref['project_domain_name'] = ref.get(
        'project_domain_name',
        uuidutils.generate_uuid(dashed=False))
    ref['roles'] = ref.get(
        'roles',
        [{'name': uuidutils.generate_uuid(dashed=False),
          'id': uuidutils.generate_uuid(dashed=False)}])
    ref['roles_link'] = ref.get('roles_link', [])
    ref['barbican_url'] = ref.get('barbican_url', BARBICAN_ENDPOINT)

    return ref


def generate_v2_project_scoped_token(**kwargs):
    """Generate a Keystone V2 token based on auth request."""
    ref = _get_normalized_token_data(**kwargs)

    o = {'access': {'token': {'id': uuidutils.generate_uuid(dashed=False),
                              'expires': '2099-05-22T00:02:43.941430Z',
                              'issued_at': '2013-05-21T00:02:43.941473Z',
                              'tenant': {'enabled': True,
                                         'id': ref.get('project_id'),
                                         'name': ref.get('project_id')
                                         }
                              },
                    'user': {'id': ref.get('user_id'),
                             'name':
                                 uuidutils.generate_uuid(dashed=False),
                             'username': ref.get('username'),
                             'roles': ref.get('roles'),
                             'roles_links': ref.get('roles_links')
                             }
                    }}

    # we only care about Barbican and Keystone endpoints
    o['access']['serviceCatalog'] = [
        {'endpoints': [
            {'publicURL': ref.get('barbican_url'),
             'id': uuidutils.generate_uuid(dashed=False),
             'region': 'RegionOne'
             }],
         'endpoints_links': [],
         'name': 'Barbican',
         'type': 'keystore'},
        {'endpoints': [
            {'publicURL': ref.get('auth_url'),
             'adminURL': ref.get('auth_url'),
             'id': uuidutils.generate_uuid(dashed=False),
             'region': 'RegionOne'
             }],
         'endpoint_links': [],
         'name': 'keystone',
         'type': 'identity'}]

    return o


def generate_v3_project_scoped_token(**kwargs):
    """Generate a Keystone V3 token based on auth request."""
    ref = _get_normalized_token_data(**kwargs)

    o = {'token': {'expires_at': '2099-05-22T00:02:43.941430Z',
                   'issued_at': '2013-05-21T00:02:43.941473Z',
                   'methods': ['password'],
                   'project': {'id': ref.get('project_id'),
                               'name': ref.get('project_name'),
                               'domain': {'id': ref.get('project_domain_id'),
                                          'name': ref.get(
                                              'project_domain_name')
                                          }
                               },
                   'user': {'id': ref.get('user_id'),
                            'name': ref.get('username'),
                            'domain': {'id': ref.get('user_domain_id'),
                                       'name': ref.get('user_domain_name')
                                       }
                            },
                   'roles': ref.get('roles')
                   }}

    # we only care about Barbican and Keystone endpoints
    o['token']['catalog'] = [
        {'endpoints': [
            {
                'id': uuidutils.generate_uuid(dashed=False),
                'interface': 'public',
                'region': 'RegionTwo',
                'url': ref.get('barbican_url')
            }],
         'id': uuidutils.generate_uuid(dashed=False),
         'type': 'keystore'},
        {'endpoints': [
            {
                'id': uuidutils.generate_uuid(dashed=False),
                'interface': 'public',
                'region': 'RegionTwo',
                'url': ref.get('auth_url')
            },
            {
                'id': uuidutils.generate_uuid(dashed=False),
                'interface': 'admin',
                'region': 'RegionTwo',
                'url': ref.get('auth_url')
            }],
         'id': uuidutils.generate_uuid(dashed=False),
         'type': 'identity'}]

    # token ID is conveyed via the X-Subject-Token header so we are generating
    # one to stash there
    token_id = uuidutils.generate_uuid(dashed=False)

    return token_id, o


class KeystoneClientFixture(testtools.TestCase):

    def setUp(self):
        super(KeystoneClientFixture, self).setUp()
        self.responses = self.useFixture(fixture.Fixture())
        self.barbican = barbicanclient.barbican.Barbican()

        self.test_arguments = {}

    def get_arguments(self, auth_version='v3'):
        if auth_version.lower() == 'v3':
            version_specific = {
                '--os-auth-url': V3_URL,
                '--os-project-name': 'my_project_name'
            }
        else:
            version_specific = {
                '--os-auth-url': V2_URL,
                '--os-identity-api-version': '2.0',
                '--os-tenant-name': 'my_tenant_name'
            }

        self.test_arguments.update(version_specific)
        return self._to_argv(self.test_arguments)

    def _to_argv(self, argument_dict):
        # Convert to argv to pass into the client
        argv = []
        for k, v in argument_dict.items():
            argv.extend([k, v])
        return argv

    def _delete_secret(self, auth_version):
        ref = '{0}/secrets/{1}'.format(BARBICAN_ENDPOINT,
                                       uuidutils.generate_uuid())

        # Mock delete secret
        self.responses.delete(ref, status_code=204)

        argv = self.get_arguments(auth_version)
        argv.extend(['--endpoint', BARBICAN_ENDPOINT, 'secret', 'delete', ref])

        try:
            self.barbican.run(argv=argv)
        except Exception:
            self.fail('failed to delete secret')

    def test_v2_auth(self):
        # Mock Keystone version discovery and token request
        self.responses.get(V2_URL, body=V2_VERSION_ENTRY)
        self.responses.post(
            '{0}/tokens'.format(V2_URL),
            json=generate_v2_project_scoped_token()
        )

        self._delete_secret('v2')

    def test_v3_auth(self):
        # Mock Keystone version discovery and token request
        self.responses.get(V3_URL, text=V3_VERSION_ENTRY)
        id, v3_token = generate_v3_project_scoped_token()

        self.responses.post(
            '{0}/auth/tokens'.format(V3_URL),
            json=v3_token,
            headers={'X-Subject-Token': '1234'}
        )

        self._delete_secret('v3')



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64

from oslo_serialization import jsonutils
from oslo_utils import timeutils

from barbicanclient import base
from barbicanclient import exceptions
from barbicanclient.tests import test_client
from barbicanclient.tests.utils import mock_get_secret_for_client
from barbicanclient.v1 import acls
from barbicanclient.v1 import secrets


class SecretData(object):
    def __init__(self):
        self.name = 'Self destruction sequence'
        self.payload = 'the magic words are squeamish ossifrage'
        self.payload_content_type = 'text/plain'
        self.algorithm = 'AES'
        self.created = str(timeutils.utcnow())
        self.consumer = {'service': 'service_test',
                         'resource_type': 'type_test',
                         'resource_id': 'id_test'}

        self.secret_dict = {'name': self.name,
                            'status': 'ACTIVE',
                            'algorithm': self.algorithm,
                            'created': self.created}

    def get_dict(self, secret_ref=None, content_types_dict=None,
                 consumers=None):
        secret = self.secret_dict
        if secret_ref:
            secret['secret_ref'] = secret_ref
        if content_types_dict:
            secret['content_types'] = content_types_dict
        if consumers:
            secret['consumers'] = consumers
        return secret


class WhenTestingSecrets(test_client.BaseEntityResource):

    def setUp(self):
        self._setUp('secrets')

        self.secret = SecretData()
        self.manager = self.client.secrets

        self.consumers_post_resource = self.entity_href + '/consumers/'
        self.consumers_delete_resource = self.entity_href + '/consumers'

    def test_should_entity_str(self):
        secret_obj = self.manager.create(name=self.secret.name)
        self.assertIn(self.secret.name, str(secret_obj))

    def test_should_entity_repr(self):
        secret_obj = self.manager.create(name=self.secret.name)
        self.assertIn('name="{0}"'.format(self.secret.name), repr(secret_obj))

    def test_should_store_via_constructor(self):
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create(name=self.secret.name,
                                     payload=self.secret.payload)
        secret_href = secret.store()
        self.assertEqual(self.entity_href, secret_href)

        # Verify that correct information was sent in the call.
        secret_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(self.secret.name, secret_req['name'])
        self.assertEqual(self.secret.payload, secret_req['payload'])

    def test_should_store_via_attributes(self):
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create()
        secret.name = self.secret.name
        secret.payload = self.secret.payload
        secret_href = secret.store()
        self.assertEqual(self.entity_href, secret_href)

        # Verify that correct information was sent in the call.
        secret_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(self.secret.name, secret_req['name'])
        self.assertEqual(self.secret.payload, secret_req['payload'])

    def test_should_store_binary_type_as_octet_stream(self):
        """We use bytes as the canonical binary type.

        The client should base64 encode the payload before sending the
        request.
        """
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        binary_payload = b'F\x130\x89f\x8e\xd9\xa1\x0e\x1f\r\xf67uu\x8b'

        secret = self.manager.create()
        secret.name = self.secret.name
        secret.payload = binary_payload
        secret.store()

        secret_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(self.secret.name, secret_req['name'])
        self.assertEqual('application/octet-stream',
                         secret_req['payload_content_type'])
        self.assertEqual('base64',
                         secret_req['payload_content_encoding'])
        self.assertNotEqual(binary_payload, secret_req['payload'])

    def test_should_store_text_type_as_text_plain(self):
        """We use unicode string as the canonical text type."""
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        text_payload = 'time for an ice cold \U0001f37a'

        secret = self.manager.create()
        secret.payload = text_payload
        secret.store()

        secret_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(text_payload, secret_req['payload'])
        self.assertEqual('text/plain', secret_req['payload_content_type'])

    def test_should_store_with_deprecated_content_type(self):
        """DEPRECATION WARNING

        Manually setting the payload_content_type is deprecated and will be
        removed in a future release.
        """
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        payload = 'I should be octet-stream'
        payload_content_type = 'text/plain'

        secret = self.manager.create()
        secret.payload = payload
        secret.payload_content_type = payload_content_type
        secret.store()

        secret_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(payload, secret_req['payload'])
        self.assertEqual(payload_content_type,
                         secret_req['payload_content_type'])

    def test_should_store_with_deprecated_content_encoding(self):
        """DEPRECATION WARNING

        Manually setting the payload_content_encoding is deprecated and will be
        removed in a future release.
        """
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        encoded_payload = base64.b64encode(
            b'F\x130\x89f\x8e\xd9\xa1\x0e\x1f\r\xf67uu\x8b'
        ).decode('UTF-8')
        payload_content_type = 'application/octet-stream'
        payload_content_encoding = 'base64'

        secret = self.manager.create()
        secret.payload = encoded_payload
        secret.payload_content_type = payload_content_type
        secret.payload_content_encoding = payload_content_encoding
        secret.store()

        secret_req = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(encoded_payload, secret_req['payload'])
        self.assertEqual(payload_content_type,
                         secret_req['payload_content_type'])
        self.assertEqual(payload_content_encoding,
                         secret_req['payload_content_encoding'])

    def test_should_be_immutable_after_submit(self):
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create(name=self.secret.name,
                                     payload=self.secret.payload)
        secret_href = secret.store()

        self.assertEqual(self.entity_href, secret_href)

        # Verify that attributes are immutable after store.
        attributes = [
            "name", "expiration", "algorithm", "bit_length", "mode",
            "payload_content_type", "payload_content_encoding", "secret_type"]
        for attr in attributes:
            try:
                setattr(secret, attr, "test")
                self.fail("didn't raise an ImmutableException exception")
            except base.ImmutableException:
                pass

    def test_should_not_be_able_to_set_generated_attributes(self):
        secret = self.manager.create()

        # Verify that generated attributes cannot be set.
        attributes = [
            "secret_ref", "created", "updated", "content_types", "status"
        ]
        for attr in attributes:
            try:
                setattr(secret, attr, "test")
                self.fail("didn't raise an AttributeError exception")
            except AttributeError:
                pass

    def test_should_get_lazy(self, secret_ref=None):
        secret_ref = secret_ref or self.entity_href

        data = self.secret.get_dict(secret_ref)
        m = self.responses.get(self.entity_href, json=data)

        secret = self.manager.get(secret_ref=secret_ref)
        self.assertIsInstance(secret, secrets.Secret)
        self.assertEqual(secret_ref, secret.secret_ref)

        # Verify GET wasn't called yet
        self.assertFalse(m.called)

        # Check an attribute to trigger lazy-load
        self.assertEqual(self.secret.name, secret.name)

        # Verify the correct URL was used to make the GET call
        self.assertEqual(self.entity_href, m.last_request.url)

    def test_should_get_lazy_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_get_lazy(bad_href)

    def test_should_get_lazy_using_only_uuid(self):
        self.test_should_get_lazy(self.entity_id)

    def test_should_get_acls_lazy(self):
        data = self.secret.get_dict(self.entity_href)
        m = self.responses.get(self.entity_href, json=data)

        acl_data = {'read': {'project-access': True, 'users': ['u1']}}
        acl_ref = self.entity_href + '/acl'
        n = self.responses.get(acl_ref, json=acl_data)

        secret = self.manager.get(secret_ref=self.entity_href)
        self.assertIsNotNone(secret)

        self.assertEqual(self.secret.name, secret.name)
        # Verify GET was called for secret but for acl it was not called
        self.assertTrue(m.called)
        self.assertFalse(n.called)

        # Check an attribute to trigger lazy-load
        self.assertEqual(['u1'], secret.acls.read.users)
        self.assertTrue(secret.acls.read.project_access)
        self.assertIsInstance(secret.acls, acls.SecretACL)

        # Verify the correct URL was used to make the GET call
        self.assertEqual(acl_ref, n.last_request.url)

    def test_should_get_payload_only_when_content_type_is_set(self):
        """DEPRECATION WARNING

        Manually setting the payload_content_type is deprecated and will be
        removed in a future release.
        """
        m = self.responses.get(self.entity_href,
                               request_headers={'Accept': 'application/json'},
                               json=self.secret.get_dict(self.entity_href))
        n = self.responses.get(self.entity_payload_href,
                               request_headers={'Accept': 'text/plain'},
                               text=self.secret.payload)

        secret = self.manager.get(
            secret_ref=self.entity_href,
            payload_content_type=self.secret.payload_content_type
        )
        self.assertIsInstance(secret, secrets.Secret)
        self.assertEqual(self.entity_href, secret.secret_ref)

        # Verify `get` wasn't called yet (metadata)
        self.assertFalse(m.called)

        # Verify `get_raw` wasn't called yet (payload)
        self.assertFalse(n.called)

        # GET payload (with payload_content_type)
        self.assertEqual(self.secret.payload, secret.payload)

        # Verify `get` still wasn't called (metadata)
        self.assertFalse(m.called)

        # Verify `get_raw` was called (payload)
        self.assertTrue(n.called)

        # Verify the correct URL was used to make the `get_raw` call
        self.assertEqual(self.entity_payload_href, n.last_request.url)

    def test_should_fetch_metadata_to_get_payload(self):
        content_types_dict = {'default': 'text/plain'}

        data = self.secret.get_dict(self.entity_href,
                                    content_types_dict=content_types_dict)
        metadata_response = self.responses.get(
            self.entity_href,
            request_headers={'Accept': 'application/json'},
            json=data)

        request_headers = {'Accept': 'text/plain'}
        decryption_response = self.responses.get(
            self.entity_payload_href,
            request_headers=request_headers,
            text=self.secret.payload)

        secret = self.manager.get(secret_ref=self.entity_href)
        self.assertIsInstance(secret, secrets.Secret)
        self.assertEqual(self.entity_href, secret.secret_ref)

        # Verify `get` wasn't called yet (metadata)
        self.assertFalse(metadata_response.called)

        # Verify `get_raw` wasn't called yet (payload)
        self.assertFalse(decryption_response.called)

        # GET payload (with no payload_content_type) trigger lazy-load
        self.assertEqual(self.secret.payload, secret.payload)

        # Verify `get` was called (metadata)
        self.assertTrue(metadata_response.called)

        # Verify `get_raw` was called (payload)
        self.assertTrue(decryption_response.called)

        # Verify the correct URL was used to make the `get` calls
        self.assertEqual(self.entity_href, metadata_response.last_request.url)
        self.assertEqual(self.entity_payload_href,
                         decryption_response.last_request.url)

    def test_should_decrypt_when_content_type_is_set(self):
        """DEPRECATION WARNING

        Manually setting the payload_content_type is deprecated and will be
        removed in a future release.
        """
        decrypted = b'decrypted text here'

        request_headers = {'Accept': 'application/octet-stream'}

        m = self.responses.get(self.entity_payload_href,
                               request_headers=request_headers,
                               content=decrypted)

        secret = self.manager.get(
            secret_ref=self.entity_href,
            payload_content_type='application/octet-stream'
        )
        secret_payload = secret.payload
        self.assertEqual(decrypted, secret_payload)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_payload_href, m.last_request.url)

    def test_should_decrypt(self, secret_ref=None):
        secret_ref = secret_ref or self.entity_href

        content_types_dict = {'default': 'application/octet-stream'}
        json = self.secret.get_dict(secret_ref, content_types_dict)
        metadata_response = self.responses.get(
            self.entity_href,
            request_headers={'Accept': 'application/json'},
            json=json)

        decrypted = b'decrypted text here'
        request_headers = {'Accept': 'application/octet-stream'}
        decryption_response = self.responses.get(
            self.entity_payload_href,
            request_headers=request_headers,
            content=decrypted)

        secret = self.manager.get(secret_ref=secret_ref)
        secret_payload = secret.payload
        self.assertEqual(decrypted, secret_payload)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, metadata_response.last_request.url)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_payload_href,
                         decryption_response.last_request.url)

    def test_should_decrypt_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_decrypt(bad_href)

    def _mock_delete_secret(self):
        self.responses.delete(self.entity_href, status_code=204)

    def _delete_from_manager(self, secret_ref, force=False):
        mock_get_secret_for_client(self.client)
        self._mock_delete_secret()
        self.manager.delete(secret_ref=secret_ref, force=force)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

    def test_should_delete_from_manager(self):
        self._delete_from_manager(self.entity_href)

    def test_should_delete_from_manager_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self._delete_from_manager(secret_ref=bad_href)

    def test_should_delete_from_manager_using_only_uuid(self):
        self._delete_from_manager(secret_ref=self.entity_id)

    def test_should_delete_from_object(self, secref_ref=None):
        secref_ref = secref_ref or self.entity_href
        data = {'secret_ref': secref_ref}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create()
        secret.payload = None
        secret.store()

        # Verify the secret has the correct ref for testing deletes
        self.assertEqual(secref_ref, secret.secret_ref)

        self.responses.delete(self.entity_href, status_code=204)

        secret.delete()

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

    def test_should_delete_from_object_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_delete_from_object(bad_href)

    def test_should_delete_from_object_using_only_uuid(self):
        self.test_should_delete_from_object(self.entity_id)

    def test_should_update_from_manager(self, secret_ref=None):
        text_payload = 'time for an ice cold \U0001f37a'
        secret_ref = secret_ref or self.entity_href

        self.responses.put(self.entity_href, status_code=204)

        self.manager.update(secret_ref=secret_ref, payload=text_payload)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

    def test_should_update_from_manager_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_update_from_manager(bad_href)

    def test_should_update_from_manager_using_only_uuid(self):
        self.test_should_update_from_manager(self.entity_id)

    def test_should_update_from_object(self, secref_ref=None):
        secref_ref = secref_ref or self.entity_href
        data = {'secret_ref': secref_ref}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create()
        secret.payload = None
        secret.store()

        # Verify the secret has the correct ref for testing updates
        self.assertEqual(secref_ref, secret.secret_ref)

        text_payload = 'time for an ice cold \U0001f37a'

        self.responses.put(self.entity_href, status_code=204)

        secret.payload = text_payload
        secret.update()

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_href, self.responses.last_request.url)

        # Verify that the data has been updated
        self.assertEqual(text_payload, secret.payload)

    def test_should_update_from_object_using_stripped_uuid(self):
        bad_href = "http://badsite.com/" + self.entity_id
        self.test_should_update_from_object(bad_href)

    def test_should_update_from_object_using_only_uuid(self):
        self.test_should_update_from_object(self.entity_id)

    def test_should_get_list(self):
        secret_resp = self.secret.get_dict(self.entity_href)

        data = {"secrets": [secret_resp for v in range(3)]}
        m = self.responses.get(self.entity_base, json=data)

        secrets_list = self.manager.list(limit=10, offset=5)
        self.assertTrue(len(secrets_list) == 3)
        self.assertIsInstance(secrets_list[0], secrets.Secret)
        self.assertEqual(self.entity_href, secrets_list[0].secret_ref)

        # Verify the correct URL was used to make the call.
        self.assertEqual(self.entity_base,
                         m.last_request.url.split('?')[0])

        # Verify that correct information was sent in the call.
        self.assertEqual(['10'], m.last_request.qs['limit'])
        self.assertEqual(['5'], m.last_request.qs['offset'])

    def test_should_fail_get_invalid_secret(self):
        self.assertRaises(ValueError, self.manager.get,
                          **{'secret_ref': '12345'})

    def test_should_fail_update_zero(self):
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create()
        secret.payload = None
        secret.store()

        self.responses.put(self.entity_href, status_code=204)
        secret.payload = 0

        # Verify that an error is thrown
        self.assertRaises(exceptions.PayloadException, secret.update)

    def test_should_fail_store_zero(self):
        data = {'secret_ref': self.entity_href}
        self.responses.post(self.entity_base + '/', json=data)

        secret = self.manager.create()
        secret.name = self.secret.name
        secret.payload = 0

        self.assertRaises(exceptions.PayloadException, secret.store)

    def test_should_fail_decrypt_no_content_types(self):
        data = self.secret.get_dict(self.entity_href)
        self.responses.get(self.entity_href, json=data)
        secret = self.manager.get(secret_ref=self.entity_href)

        self.assertIsNone(secret.payload)

    def test_should_fail_decrypt_no_default_content_type(self):
        content_types_dict = {'no-default': 'application/octet-stream'}
        data = self.secret.get_dict(self.entity_href, content_types_dict)
        self.responses.get(self.entity_href, json=data)

        secret = self.manager.get(secret_ref=self.entity_href)
        self.assertIsNone(secret.payload)

    def test_should_fail_delete_no_href(self):
        self.assertRaises(ValueError, self.manager.delete, None)

    def test_should_get_total(self):
        self.responses.get(self.entity_base, json={'total': 1})
        total = self.manager.total()
        self.assertEqual(1, total)

    def test_get_formatted_data(self):
        data = self.secret.get_dict(self.entity_href)
        self.responses.get(self.entity_href, json=data)

        secret = self.manager.get(secret_ref=self.entity_href)
        f_data = secret._get_formatted_data()
        self.assertEqual(
            timeutils.parse_isotime(data['created']).isoformat(),
            f_data[2])


class WhenTestingSecretMetadata(test_client.BaseEntityResource):

    def setUp(self):
        self._setUp('secrets')
        self.secret = SecretData()
        self.manager = self.client.secrets
        self.metadata_base = self.entity_href + '/metadata'

    def test_should_get_all_metadata(self):
        """Test GET /v1/secrets/{uuid}/metadata"""
        metadata_response = {
            'metadata': {
                'description': 'contains the AES key',
                'geolocation': '12.3456, -98.7654'
            }
        }
        m = self.responses.get(self.metadata_base, json=metadata_response)

        result = self.manager.get_secret_metadata(self.entity_href)

        self.assertEqual(metadata_response, result)
        self.assertEqual(self.metadata_base, m.last_request.url)

    def test_should_get_specific_metadata_key(self):
        """Test GET /v1/secrets/{uuid}/metadata/{key}"""
        key = 'description'
        metadata_key_response = {
            'key': 'description',
            'value': 'contains the AES key'
        }
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.get(metadata_key_url, json=metadata_key_response)

        result = self.manager.get_secret_metadata(self.entity_href, key=key)

        self.assertEqual(metadata_key_response, result)
        self.assertEqual(metadata_key_url, m.last_request.url)

    def test_should_get_metadata_using_stripped_uuid(self):
        """Test with stripped UUID"""
        bad_href = "http://badsite.com/" + self.entity_id
        metadata_response = {'metadata': {'test': 'value'}}
        m = self.responses.get(self.metadata_base, json=metadata_response)

        result = self.manager.get_secret_metadata(bad_href)

        self.assertEqual(metadata_response, result)
        self.assertEqual(self.metadata_base, m.last_request.url)

    def test_should_get_metadata_using_only_uuid(self):
        """Test with UUID only"""
        metadata_response = {'metadata': {'test': 'value'}}
        m = self.responses.get(self.metadata_base, json=metadata_response)

        result = self.manager.get_secret_metadata(self.entity_id)

        self.assertEqual(metadata_response, result)
        self.assertEqual(self.metadata_base, m.last_request.url)

    def test_should_set_metadata(self):
        """Test PUT /v1/secrets/{uuid}/metadata"""
        metadata = {
            'description': 'updated description',
            'environment': 'production'
        }
        set_response = {
            'metadata_ref': self.entity_href + '/metadata'
        }
        m = self.responses.put(self.metadata_base, json=set_response)

        result = self.manager.set_secret_metadata(self.entity_href, metadata)

        self.assertEqual(set_response, result)
        self.assertEqual(self.metadata_base, m.last_request.url)

        request_data = jsonutils.loads(m.last_request.text)
        self.assertEqual({'metadata': metadata}, request_data)

    def test_should_set_metadata_using_stripped_uuid(self):
        """Test PUT with stripped UUID"""
        bad_href = "http://badsite.com/" + self.entity_id
        metadata = {'test': 'value'}
        set_response = {'metadata_ref': self.entity_href + '/metadata'}
        m = self.responses.put(self.metadata_base, json=set_response)

        result = self.manager.set_secret_metadata(bad_href, metadata)

        self.assertEqual(set_response, result)
        self.assertEqual(self.metadata_base, m.last_request.url)

    def test_should_set_metadata_using_only_uuid(self):
        """Test PUT with UUID only"""
        metadata = {'test': 'value'}
        set_response = {'metadata_ref': self.entity_href + '/metadata'}
        m = self.responses.put(self.metadata_base, json=set_response)

        result = self.manager.set_secret_metadata(self.entity_id, metadata)

        self.assertEqual(set_response, result)
        self.assertEqual(self.metadata_base, m.last_request.url)

    def test_should_add_metadata_key(self):
        """Test POST /v1/secrets/{uuid}/metadata/{key}"""
        key = 'owner'
        value = 'admin@example.com'
        add_response = {
            'metadata_ref': self.entity_href + '/metadata/' + key
        }
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.post(metadata_key_url, json=add_response)

        result = self.manager.add_secret_metadata(
            self.entity_href, key, value)

        self.assertEqual(add_response, result)
        self.assertEqual(metadata_key_url, m.last_request.url)

        request_data = jsonutils.loads(m.last_request.text)
        expected_payload = {'key': key, 'value': value}
        self.assertEqual(expected_payload, request_data)

    def test_should_add_metadata_key_using_stripped_uuid(self):
        """Test POST with stripped UUID"""
        bad_href = "http://badsite.com/" + self.entity_id
        key = 'test'
        value = 'value'
        add_response = {
            'metadata_ref': self.entity_href + '/metadata/' + key
        }
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.post(metadata_key_url, json=add_response)

        result = self.manager.add_secret_metadata(bad_href, key, value)

        self.assertEqual(add_response, result)
        self.assertEqual(metadata_key_url, m.last_request.url)

    def test_should_add_metadata_key_using_only_uuid(self):
        """Test POST with UUID only"""
        key = 'test'
        value = 'value'
        add_response = {
            'metadata_ref': self.entity_href + '/metadata/' + key
        }
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.post(metadata_key_url, json=add_response)

        result = self.manager.add_secret_metadata(self.entity_id, key, value)

        self.assertEqual(add_response, result)
        self.assertEqual(metadata_key_url, m.last_request.url)

    def test_should_delete_metadata_key(self):
        """Test DELETE /v1/secrets/{uuid}/metadata/{key}"""
        key = 'description'
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.delete(metadata_key_url, status_code=204)

        self.manager.delete_secret_metadata(self.entity_href, key)

        self.assertEqual(metadata_key_url, m.last_request.url)

    def test_should_delete_metadata_key_using_stripped_uuid(self):
        """Test DELETE with stripped UUID"""
        bad_href = "http://badsite.com/" + self.entity_id
        key = 'test'
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.delete(metadata_key_url, status_code=204)

        self.manager.delete_secret_metadata(bad_href, key)

        self.assertEqual(metadata_key_url, m.last_request.url)

    def test_should_delete_metadata_key_using_only_uuid(self):
        """Test DELETE with UUID only"""
        key = 'test'
        metadata_key_url = self.metadata_base + '/' + key
        m = self.responses.delete(metadata_key_url, status_code=204)

        self.manager.delete_secret_metadata(self.entity_id, key)

        self.assertEqual(metadata_key_url, m.last_request.url)

    def test_should_fail_get_metadata_invalid_secret(self):
        self.assertRaises(
            ValueError,
            self.manager.get_secret_metadata,
            **{'secret_ref': '12345'}
        )

    def test_should_fail_set_metadata_invalid_secret(self):
        self.assertRaises(
            ValueError,
            self.manager.set_secret_metadata,
            **{'secret_ref': '12345', 'metadata': {}}
        )

    def test_should_fail_add_metadata_invalid_secret(self):
        self.assertRaises(
            ValueError,
            self.manager.add_secret_metadata,
            **{'secret_ref': '12345', 'key': 'test', 'value': 'value'}
        )

    def test_should_fail_delete_metadata_invalid_secret(self):
        self.assertRaises(
            ValueError,
            self.manager.delete_secret_metadata,
            **{'secret_ref': '12345', 'key': 'test'}
        )



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import datetime
import sys

from functionaltests.client import base
from functionaltests.common import cleanup
from functionaltests import utils
from oslo_utils import timeutils
from testtools import testcase

from barbicanclient import exceptions

order_create_key_data = {
    "name": "barbican functional test secret name",
    "algorithm": "aes",
    "bit_length": 256,
    "mode": "cbc",
    "payload_content_type": "application/octet-stream",
}

# Any field with None will be created in the model with None as the value
# but will be omitted in the final request (via the requests package)
# to the server.
#
# Given that fact, order_create_nones_data is effectively an empty json request
# to the server.
order_create_nones_data = {
    "name": None,
    "algorithm": None,
    "bit_length": None,
    "mode": None,
    "payload_content_type": None,
}


@utils.parameterized_test_case
class OrdersTestCase(base.TestCase):

    def setUp(self):
        super(OrdersTestCase, self).setUp()
        self.cleanup = cleanup.CleanUp(self.barbicanclient)

    def tearDown(self):
        self.cleanup.delete_all_entities()
        super(OrdersTestCase, self).tearDown()

    @testcase.attr('positive')
    def test_create_order_defaults_wout_name(self):
        """Create an order without the name attribute."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.name = None
        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.name, order_resp.name)

    @testcase.attr('positive')
    def test_create_order_defaults_w_empty_name(self):
        """Create an order the name attribute an empty string."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.name = ""
        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.name, order_resp.name)

    @testcase.skip('Launchpad 1425667')
    @testcase.attr('positive')
    def test_create_order_defaults_payload_content_type_none(self):
        """Covers creating orders with various valid payload content types."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.payload_content_type = None

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertTrue(order_resp.status == "ACTIVE" or
                        order_resp.status == "PENDING")

    @testcase.attr('positive')
    def test_create_order_defaults_check_empty_name(self):
        """Create order with empty meta name.

        The resulting secret name should be a UUID.
        """

        # first create an order with defaults
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.name = ""

        order_ref = self.cleanup.add_entity(order)

        # verify that the order was created successfully
        self.assertIsNotNone(order_ref)

        # given the order href, retrieve the order
        order_resp = self.barbicanclient.orders.get(order_ref)

        # verify that the get was successful
        self.assertTrue(order_resp.status == "ACTIVE" or
                        order_resp.status == "PENDING")

        # verify the new secret's name matches the name in the secret ref
        # in the newly created order.
        secret_resp = self.barbicanclient.secrets.get(order_resp.secret_ref)
        self.assertEqual(order.name, secret_resp.name)

    @testcase.attr('positive')
    def test_order_and_secret_metadata_same(self):
        """Checks that metadata from secret GET and order GET are the same.

        Covers checking that secret metadata from a get on the order and
        secret metadata from a get on the secret are the same. Assumes
        that the order status will be active and not pending.
        """
        order = self.barbicanclient.orders.create_key(**order_create_key_data)

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertIsNotNone(order_resp.secret_ref)

        secret_resp = self.barbicanclient.secrets.get(order_resp.secret_ref)

        self.assertEqual(secret_resp.name,
                         order_resp.name,
                         'Names were not the same')
        self.assertEqual(secret_resp.algorithm,
                         order_resp.algorithm,
                         'Algorithms were not the same')
        self.assertEqual(secret_resp.bit_length,
                         order_resp.bit_length,
                         'Bit lengths were not the same')
        self.assertEqual(secret_resp.expiration,
                         order_resp.expiration,
                         'Expirations were not the same')
        self.assertEqual(secret_resp.mode,
                         order_resp.mode,
                         'Modes were not the same')

    @testcase.attr('negative')
    def test_get_order_defaults_that_doesnt_exist(self):
        """Covers case of getting a non-existent order."""
        ref = self.barbicanclient.orders._api.endpoint_override + \
            '/orders/notauuid'
        # try to get a non-existent order
        e = self.assertRaises(ValueError, self.barbicanclient.orders.get, ref)

        # verify that the order get failed
        self.assertEqual('Order incorrectly specified.', str(e))

    @testcase.attr('negative')
    def test_get_order_defaults_that_doesnt_exist_valid_uuid(self):
        """Covers case of getting a non-existent order with a valid UUID"""
        uuid = '54262d9d-4bc7-4821-8df0-dc2ca8e112bb'
        ref = self.barbicanclient.orders._api.endpoint_override + \
            '/orders/' + uuid

        # try to get a non-existent order
        e = self.assertRaises(
            exceptions.HTTPClientError,
            self.barbicanclient.orders.get,
            ref
        )

        # verify that the order get failed
        self.assertEqual(404, e.status_code)

    @testcase.attr('negative')
    def test_create_order_nones(self):
        """Covers order creation with empty JSON."""
        order = self.barbicanclient.orders.create_key(
            **order_create_nones_data)

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)

    @testcase.attr('negative')
    def test_create_order_empty_entries(self):
        """Covers order creation with empty JSON."""
        order = self.barbicanclient.orders.create_key(
            **order_create_nones_data)
        order.name = ""
        order.algorithm = ""
        order.mode = ""
        order.bit_length = ""
        order.payload_content_type = ""

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)

    @testcase.attr('negative')
    def test_create_order_defaults_oversized_strings(self):
        """Covers order creation with empty JSON."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.name = base.TestCase.oversized_field
        order.algorithm = base.TestCase.oversized_field
        order.mode = base.TestCase.oversized_field

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        '8': [8],
        '64': [64],
        '128': [128],
        '192': [192],
        '256': [256],
        '1024': [1024],
        '2048': [2048],
        '4096': [4096]
    })
    @testcase.attr('positive')
    def test_create_order_defaults_valid_bit_length(self, bit_length):
        """Covers creating orders with various valid bit lengths."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.bit_length = bit_length

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.bit_length, order_resp.bit_length)

    @utils.parameterized_dataset({
        'negative_maxsize': [-sys.maxsize],
        'negative_7': [-7],
        'negative_1': [-1],
        '0': [0],
        '1': [1],
        '7': [7],
        '129': [129],
        'none': [None],
        'empty': [''],
        'space': [' '],
        'over_signed_small_int': [32768]
    })
    @testcase.attr('negative')
    def test_create_order_defaults_invalid_bit_length(self, bit_length):
        """Covers creating orders with various invalid bit lengths."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.bit_length = bit_length

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )
        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'alphanumeric': ['1f34ds'],
        'len_255': [base.TestCase.max_sized_field],
        'uuid': ['54262d9d-4bc7-4821-8df0-dc2ca8e112bb'],
        'punctuation': ['~!@#$%^&*()_+`-={}[]|:;<>,.?']
    })
    @testcase.attr('positive')
    def test_create_order_defaults_valid_name(self, name):
        """Covers creating orders with various valid names."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.name = name

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.name, order_resp.name)

    @utils.parameterized_dataset({
        'int': [123]
    })
    @testcase.attr('negative')
    def test_create_order_defaults_invalid_name(self, name):
        """Covers creating orders with various invalid names."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.name = name

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'cbc': ['cbc']
    })
    @testcase.attr('positive')
    def test_create_order_defaults_valid_mode(self, mode):
        """Covers creating orders with various valid modes."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.mode = mode

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.mode, order_resp.mode)

    @utils.parameterized_dataset({
        'int': [123]
    })
    @testcase.attr('negative')
    def test_create_order_defaults_invalid_mode(self, mode):
        """Covers creating orders with various invalid modes."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.mode = mode

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )
        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'aes': ['aes']
    })
    @testcase.attr('positive')
    def test_create_order_defaults_valid_algorithm(self, algorithm):
        """Covers creating orders with various valid algorithms."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.algorithm = algorithm

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.algorithm, order_resp.algorithm)

    @utils.parameterized_dataset({
        'int': [123]
    })
    @testcase.attr('negative')
    def test_create_order_defaults_invalid_algorithm(self, algorithm):
        """Covers creating orders with various invalid algorithms."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.algorithm = algorithm

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)

    # TODO(tdink) Add empty after Launchpad 1420444 is resolved
    @utils.parameterized_dataset({
        'text/plain': ['text/plain'],
        'text_plain_space_charset_utf8': ['text/plain; charset=utf-8'],
    })
    @testcase.attr('positive')
    def test_create_order_defaults_valid_payload_content_type(self, pct):
        """Covers order creation with various valid payload content types."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.payload_content_type = pct

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertEqual(order.payload_content_type,
                         order_resp.payload_content_type)

    @utils.parameterized_dataset({
        'int': [123],
        'invalid': ['invalid'],
        'oversized_string': [base.TestCase.oversized_field],
        'text': ['text'],
        'text_slash_with_no_subtype': ['text/'],
    })
    @testcase.attr('negative')
    def test_create_order_defaults_invalid_payload_content_type(self, pct):
        """Covers order creation with various invalid payload content types."""
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.payload_content_type = pct

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)

    @utils.parameterized_dataset({
        'negative_five_long_expire': {
            'timezone': '-05:00',
            'days': 5},

        'positive_five_long_expire': {
            'timezone': '+05:00',
            'days': 5},

        'negative_one_short_expire': {
            'timezone': '-01',
            'days': 1},

        'positive_one_short_expire': {
            'timezone': '+01',
            'days': 1}
    })
    @testcase.attr('positive')
    def test_create_order_defaults_valid_expiration(self, **kwargs):
        """Covers creating orders with various valid expiration data."""
        timestamp = utils.create_timestamp_w_tz_and_offset(**kwargs)

        date = timeutils.parse_isotime(timestamp)
        date = date.astimezone(datetime.timezone.utc)

        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.expiration = timestamp

        order_ref = self.cleanup.add_entity(order)
        self.assertIsNotNone(order_ref)

        order_resp = self.barbicanclient.orders.get(order_ref)
        self.assertIsNotNone(order_resp)
        self.assertEqual(date, order_resp.expiration)

    @utils.parameterized_dataset({
        'malformed_timezone': {
            'timezone': '-5:00',
            'days': 5},
    })
    @testcase.attr('negative')
    def test_create_order_defaults_invalid_expiration(self, **kwargs):
        """Covers creating orders with various invalid expiration data."""
        timestamp = utils.create_timestamp_w_tz_and_offset(**kwargs)
        order = self.barbicanclient.orders.create_key(**order_create_key_data)
        order.expiration = timestamp

        e = self.assertRaises(
            exceptions.HTTPClientError,
            order.submit
        )

        self.assertEqual(400, e.status_code)



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from barbicanclient import client
from barbicanclient import exceptions
from barbicanclient.tests import test_client
from barbicanclient.tests.utils import mock_delete_secret_for_responses
from barbicanclient.tests.utils import mock_get_secret_for_client
from barbicanclient.tests.v1.test_secrets import SecretData
from barbicanclient.v1 import secrets

from oslo_serialization import jsonutils


class WhenTestingConsumers(test_client.BaseEntityResource):

    def setUp(self):
        self._setUp('secrets')

        self.secret = SecretData()

        self.client_v1_0 = client.Client(
            endpoint=self.endpoint, project_id=self.project_id,
            microversion='1.0')

        self.manager = self.client.secrets
        self.manager_v1_0 = self.client_v1_0.secrets

        self.consumers_post_resource = self.entity_href + '/consumers/'
        self.consumers_delete_resource = self.entity_href + '/consumers'

    def test_register_consumer_fails_with_lower_microversion(self):
        self.assertRaises(
            NotImplementedError, self.manager_v1_0.register_consumer,
            self.entity_href, self.secret.consumer.get('service'),
            self.secret.consumer.get('resource_type'),
            self.secret.consumer.get('resource_id'))

    def _register_consumer(self):
        data = self.secret.get_dict(
            self.entity_href, consumers=[self.secret.consumer])
        self.responses.post(self.entity_href + '/consumers/', json=data)
        return self.manager.register_consumer(
            self.entity_href, self.secret.consumer.get('service'),
            self.secret.consumer.get('resource_type'),
            self.secret.consumer.get('resource_id'))

    def test_should_register_consumer_with_correct_microversion(self):
        self._register_consumer()

    def test_should_register_consumer_and_return_secret(self):
        self.assertIsInstance(self._register_consumer(), secrets.Secret)

    def test_should_register_consumer_with_correct_secret_href(self):
        secret = self._register_consumer()
        self.assertEqual(self.entity_href, secret.secret_ref)

    def test_should_register_consumer_with_correct_url(self):
        self._register_consumer()
        self.assertEqual(
            self.consumers_post_resource, self.responses.last_request.url)

    def test_should_register_consumer_with_consumer(self):
        secret = self._register_consumer()
        self.assertEqual([self.secret.consumer], secret.consumers)

    def test_remove_consumer_fails_with_lower_microversion(self):
        self.assertRaises(
            NotImplementedError, self.manager_v1_0.remove_consumer,
            self.entity_href, self.secret.consumer.get('service'),
            self.secret.consumer.get('resource_type'),
            self.secret.consumer.get('resource_id'))

    def _remove_consumer(self):
        self.responses.delete(self.entity_href + '/consumers', status_code=204)
        self.manager.remove_consumer(
            self.entity_href, self.secret.consumer.get('service'),
            self.secret.consumer.get('resource_type'),
            self.secret.consumer.get('resource_id'))

    def test_should_remove_consumer_with_correct_microversion(self):
        self._remove_consumer()

    def test_should_remove_consumer_with_correct_url(self):
        self._remove_consumer()
        self.assertEqual(
            self.consumers_delete_resource, self.responses.last_request.url)

    def test_should_remove_consumer_with_correct_consumer(self):
        self._remove_consumer()
        self.assertEqual(
            self.consumers_delete_resource, self.responses.last_request.url)

        body = jsonutils.loads(self.responses.last_request.text)
        self.assertEqual(self.secret.consumer, body)

    def _delete_from_manager(self, secret_ref, force=False, consumers=[]):
        mock_get_secret_for_client(self.client, consumers=consumers)
        mock_delete_secret_for_responses(self.responses, self.entity_href)
        self.manager.delete(secret_ref=secret_ref, force=force)

    def _delete_from_manager_with_consumers(self, secret_ref, force=False):
        consumers = [{'service': 'service_test',
                      'resource_type': 'type_test',
                      'resource_id': 'id_test'}]

        self._delete_from_manager(secret_ref, force=force, consumers=consumers)

    def test_delete_from_manager_fails_with_consumers_without_force(self):
        self.assertRaises(
            exceptions.SecretHasConsumers,
            self._delete_from_manager_with_consumers, self.entity_href,
            force=False)

    def test_should_delete_from_manager_with_consumers_and_force(self):
        self._delete_from_manager_with_consumers(self.entity_href, force=True)

    def test_should_delete_from_manager_without_consumers_and_force(self):
        self._delete_from_manager(self.entity_href, force=True)

    def _list_consumers(self, secret_ref, consumers=[]):
        mock_get_secret_for_client(self.client, consumers)
        return self.manager.list_consumers(secret_ref)

    def test_list_consumers_from_secret_without_consumers(self):
        consumer_list = self._list_consumers(self.entity_href)
        self.assertTrue(len(consumer_list) == 0)

    def test_list_consumers_from_secret_with_consumers(self):
        consumers = [{'service': 'service_test1',
                      'resource_type': 'type_test1',
                      'resource_id': 'id_test1'},
                     {'service': 'service_test2',
                      'resource_type': 'type_test2',
                      'resource_id': 'id_test2'}]
        consumer_list = self._list_consumers(self.entity_href, consumers)

        for elem in range(len(consumers)):
            self.assertTrue(
                consumer_list[elem].service ==
                consumers[elem]['service'])
            self.assertTrue(
                consumer_list[elem].resource_type ==
                consumers[elem]['resource_type'])
            self.assertTrue(
                consumer_list[elem].resource_id ==
                consumers[elem]['resource_id'])

    def test_should_fail_list_consumers_invalid_secret(self):
        self.assertRaises(ValueError, self.manager.list_consumers,
                          **{'secret_ref': '12345'})



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import io

from requests_mock.contrib import fixture
import testtools

from barbicanclient import barbican as barb
from barbicanclient.barbican import Barbican
from barbicanclient import client
from barbicanclient import exceptions
from barbicanclient.tests import keystone_client_fixtures


class WhenTestingBarbicanCLI(testtools.TestCase):

    def setUp(self):
        super().setUp()
        self.endpoint = 'http://localhost:9311/'
        self.project_id = '1234567890abcdef1234567890abcdef'
        self.responses = self.useFixture(fixture.Fixture())
        self.responses.get(
            'http://localhost:9311/v1/',
            json={
                'version': {
                    'id': 'v1',
                    'status': 'CURRENT',
                    'min_version': '1.0',
                    'max_version': '1.1',
                    'links': [{
                        'rel': 'self',
                        'href': 'http://192.168.122.110/key-manager/v1/'
                    }, {
                        'rel': 'describedby',
                        'type': 'text/html',
                        'href': 'https://docs.openstack.org/'}]}})
        self.responses.get(
            'http://localhost:9311/',
            json={
                "versions": [{
                    "id": "v1",
                    "status": "CURRENT",
                    "min_version": "1.0",
                    "max_version": "1.1",
                    "links": [{
                        "rel": "self",
                        "href": "http://localhost:9311/v1/"
                    }, {
                        "rel": "describedby",
                        "type": "text/html",
                        "href": "https://docs.openstack.org/"}]}]})

        self.captured_stdout = io.StringIO()
        self.captured_stderr = io.StringIO()
        self.barbican = Barbican(
            stdout=self.captured_stdout,
            stderr=self.captured_stderr
        )
        self.parser = self.barbican.build_option_parser('desc', 'vers')

    def assert_client_raises(self, args, message):
        argv, remainder = self.parser.parse_known_args(args.split())
        e = self.assertRaises(
            Exception, self.barbican.create_client, argv
        )
        self.assertIn(message, str(e))

    def create_and_assert_client(self, args):
        argv, remainder = self.parser.parse_known_args(args.split())

        client = self.barbican.create_client(argv)
        self.assertIsNotNone(client)
        return client

    def test_should_show_usage_with_help_flag(self):
        self.assertRaises(SystemExit, self.barbican.run, ['-h'])
        self.assertIn('usage', self.captured_stdout.getvalue())

    def test_should_show_usage_with_no_args(self):
        exit_code = self.barbican.run([])
        self.assertEqual(1, exit_code)
        self.assertIn('usage', self.captured_stderr.getvalue())

    def test_should_error_if_noauth_and_authurl_both_specified(self):
        args = "--no-auth --os-auth-url http://localhost:5000/v3"
        message = (
            'ERROR: argument --os-auth-url/-A: not allowed with '
            'argument --no-auth/-N'
        )
        self.assert_client_raises(args, message)

    def _expect_error_with_invalid_noauth_args(self, args):
        expected_err_msg = (
            'ERROR: please specify --endpoint '
            'and --os-project-id (or --os-tenant-id)'
        )
        self.assert_client_raises(args, expected_err_msg)

    def test_should_error_if_noauth_and_missing_endpoint_tenantid_args(self):
        self._expect_error_with_invalid_noauth_args("--no-auth secret list")
        self._expect_error_with_invalid_noauth_args(
            "--no-auth --endpoint http://xyz secret list")
        self._expect_error_with_invalid_noauth_args(
            "--no-auth --os-tenant-id 123 secret list")
        self._expect_error_with_invalid_noauth_args(
            "--no-auth --os-project-id 123 secret list")

    def test_should_succeed_if_noauth_with_valid_args_specified(self):
        args = (
            '--no-auth --endpoint {0} --os-tenant-id {1}'
            'secret list'.format(self.endpoint, self.project_id)
        )
        list_secrets_url = '{0}/v1/secrets'.format(self.endpoint.rstrip('/'))
        self.responses.get(list_secrets_url, json={"secrets": [], "total": 0})
        client = self.create_and_assert_client(args)
        secret_list = client.secrets.list()
        self.assertTrue(self.responses._adapter.called)
        # there should be two requests
        # 1. requests sent by microversions check
        # 2. the request under test.
        self.assertEqual(2, self.responses._adapter.call_count)
        self.assertEqual([], secret_list)

    def test_should_error_if_required_keystone_auth_arguments_are_missing(
            self):
        expected_error_msg = (
            'ERROR: please specify the following --os-project-id or'
            ' (--os-project-name and --os-project-domain-name) or '
            ' (--os-project-name and --os-project-domain-id)'
        )
        self.assert_client_raises(
            '--os-auth-url http://localhost:35357/v2.0 secret list',
            expected_error_msg)
        self.assert_client_raises(
            '--os-auth-url http://localhost:35357/v2.0 --os-username barbican '
            '--os-password barbican secret list',
            expected_error_msg
        )

    def test_check_auth_arguments_v2(self):
        args = ("--os-username 'bob' --os-password 'jan' --os-auth-url 'boop'"
                " --os-tenant-id 123 --os-identity-api-version '2.0'")
        argv, remainder = self.parser.parse_known_args(args.split())
        api_version = argv.os_identity_api_version
        barbican = Barbican()
        response = barbican.check_auth_arguments(argv, api_version)
        self.assertTrue(response)

    def test_should_fail_check_auth_arguments_v2(self):
        args = ("--os-username bob --os-password jan --os-auth-url boop"
                " --os-identity-api-version 2.0")
        message = 'ERROR: please specify --os-tenant-id or --os-tenant-name'
        argv, remainder = self.parser.parse_known_args(args.split())
        api_version = argv.os_identity_api_version
        e = self.assertRaises(
            Exception,
            self.barbican.check_auth_arguments,
            argv,
            api_version,
            True
        )
        self.assertIn(message, str(e))

    def test_should_fail_create_client_with_no_auth_url(self):
        args = '--os-auth-token 1234567890 --os-tenant-id 123'
        message = 'ERROR: please specify --os-auth-url'
        argv, remainder = self.parser.parse_known_args(args.split())
        e = self.assertRaises(
            Exception, self.barbican.create_client, argv
        )
        self.assertIn(message, str(e))

    def test_should_fail_missing_credentials(self):
        message = 'ERROR: please specify authentication credentials'
        args = ''
        argv, remainder = self.parser.parse_known_args(args.split())
        e = self.assertRaises(
            Exception, self.barbican.create_client, argv
        )
        self.assertIn(message, str(e))

    def test_main(self):
        args = ''
        response = barb.main(args)
        self.assertEqual(1, response)

    def test_default_endpoint_filter_kwargs_set_correctly(self):
        auth_args = ('--no-auth --endpoint http://localhost:9311/ '
                     '--os-project-id project1')
        argv, remainder = self.parser.parse_known_args(auth_args.split())
        barbican_client = self.barbican.create_client(argv)
        httpclient = barbican_client.secrets._api

        self.assertEqual(client._DEFAULT_SERVICE_INTERFACE,
                         httpclient.interface)
        self.assertEqual(client._DEFAULT_SERVICE_TYPE, httpclient.service_type)
        self.assertEqual(client._DEFAULT_API_VERSION, httpclient.version)
        self.assertIsNone(httpclient.service_name)

    def test_endpoint_filter_kwargs_set_correctly(self):
        auth_args = ('--no-auth --endpoint http://localhost:9311/ '
                     '--os-project-id project1')
        endpoint_filter_args = ('--interface private '
                                '--service-type custom-type '
                                '--service-name Burrbican '
                                '--region-name RegionTwo '
                                '--barbican-api-version v1')
        args = auth_args + ' ' + endpoint_filter_args
        argv, remainder = self.parser.parse_known_args(args.split())
        barbican_client = self.barbican.create_client(argv)
        httpclient = barbican_client.secrets._api

        self.assertEqual('private', httpclient.interface)
        self.assertEqual('custom-type', httpclient.service_type)
        self.assertEqual('Burrbican', httpclient.service_name)
        self.assertEqual('RegionTwo', httpclient.region_name)
        self.assertEqual('v1', httpclient.version)

    def test_should_fail_if_provide_unsupported_api_version(self):
        auth_args = ('--no-auth --endpoint http://localhost:9311/ '
                     '--os-project-id project1')
        endpoint_filter_args = ('--interface private '
                                '--service-type custom-type '
                                '--service-name Burrbican '
                                '--region-name RegionTwo '
                                '--barbican-api-version v2')
        args = auth_args + ' ' + endpoint_filter_args
        argv, remainder = self.parser.parse_known_args(args.split())

        self.assertRaises(exceptions.UnsupportedVersion,
                          self.barbican.create_client,
                          argv)

    def test_should_prevent_mutual_exclusive_file_opt(self):
        args = (
            '--no-auth --endpoint {0} --os-tenant-id {1}'
            '--file foo --payload'
            'secret get'.format(self.endpoint, self.project_id)
        )
        list_secrets_url = '{0}/v1/secrets'.format(self.endpoint.rstrip('/'))
        self.responses.get(list_secrets_url, json={"secrets": [], "total": 0})
        client = self.create_and_assert_client(args)
        secret_list = client.secrets.list()
        self.assertTrue(self.responses._adapter.called)
        self.assertEqual(2, self.responses._adapter.call_count)
        self.assertEqual([], secret_list)

    def test_insecure_true_kwargs_set_correctly(self):
        auth_args = ('--no-auth --endpoint http://localhost:9311/ '
                     '--os-project-id project1')
        endpoint_filter_args = ('--interface public '
                                '--service-type custom-type '
                                '--service-name Burrbican '
                                '--region-name RegionTwo '
                                '--barbican-api-version v1')
        args = auth_args + ' ' + endpoint_filter_args
        argv, remainder = self.parser.parse_known_args(args.split())
        argv.insecure = True
        argv.os_identity_api_version = '2.0'
        argv.os_tenant_name = 'my_tenant_name'
        barbican_client = self.barbican.create_client(argv)
        httpclient = barbican_client.secrets._api
        self.assertFalse(httpclient.session.verify)


class TestBarbicanWithKeystonePasswordAuth(
        keystone_client_fixtures.KeystoneClientFixture):

    def setUp(self):
        super(TestBarbicanWithKeystonePasswordAuth, self).setUp()

        self.test_arguments = {
            '--os-username': 'some_user',
            '--os-password': 'some_pass',
        }


class TestBarbicanWithKeystoneTokenAuth(
        keystone_client_fixtures.KeystoneClientFixture):

    def setUp(self):
        super(TestBarbicanWithKeystoneTokenAuth, self).setUp()

        self.test_arguments = {
            '--os-auth-token': 'some_token',
            '--os-project-id': 'some_project_id',
        }



# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from keystoneauth1 import session
from requests_mock.contrib import fixture
import testtools

from barbicanclient import client
from barbicanclient import exceptions
from barbicanclient.exceptions import UnsupportedVersion
from barbicanclient.tests.utils import get_server_supported_versions
from barbicanclient.tests.utils import get_version_endpoint
from barbicanclient.tests.utils import mock_session
from barbicanclient.tests.utils import mock_session_get
from barbicanclient.tests.utils import mock_session_get_endpoint


_DEFAULT_MICROVERSION = (1, 1)


class TestClient(testtools.TestCase):

    def setUp(self):
        super(TestClient, self).setUp()
        self.endpoint = 'http://localhost:9311'
        self.responses = self.useFixture(fixture.Fixture())
        self.responses.get(
            'http://localhost:9311/v1/',
            json={
                'version': {
                    'id': 'v1',
                    'status': 'CURRENT',
                    'min_version': '1.0',
                    'max_version': '1.1',
                    'links': [{
                        'rel': 'self',
                        'href': 'http://192.168.122.110/key-manager/v1/'
                    }, {
                        'rel': 'describedby',
                        'type': 'text/html',
                        'href': 'https://docs.openstack.org/'}]}})
        self.responses.get(
            'http://localhost:9311/',
            json={
                "versions": [{
                    "id": "v1",
                    "status": "CURRENT",
                    "min_version": "1.0",
                    "max_version": "1.1",
                    "links": [{
                        "rel": "self",
                        "href": "http://localhost:9311/v1/"
                    }, {
                        "rel": "describedby",
                        "type": "text/html",
                        "href": "https://docs.openstack.org/"}]}]})

        self.project_id = 'project_id'
        self.session = session.Session()
        self.httpclient = client._HTTPClient(
            session=self.session,
            microversion=_DEFAULT_MICROVERSION,
            endpoint=self.endpoint,
            project_id=self.project_id)


class WhenTestingClientInit(TestClient):

    def test_api_version_is_appended_to_endpoint(self):
        c = client.Client(session=self.session,
                          endpoint=self.endpoint,
                          project_id=self.project_id)
        self.assertEqual('http://localhost:9311/v1/',
                         c.client.endpoint_override)

    def test_default_headers_are_empty(self):
        c = client._HTTPClient(
            session=self.session, microversion='1.1', endpoint=self.endpoint)
        self.assertIsInstance(c._default_headers, dict)
        self.assertFalse(bool(c._default_headers))

    def test_project_id_is_added_to_default_headers(self):
        c = client._HTTPClient(session=self.session,
                               microversion=_DEFAULT_MICROVERSION,
                               endpoint=self.endpoint,
                               project_id=self.project_id)
        self.assertIn('X-Project-Id', c._default_headers.keys())
        self.assertEqual(self.project_id, c._default_headers['X-Project-Id'])

    def test_error_thrown_when_no_session_and_no_endpoint(self):
        self.assertRaises(ValueError, client.Client,
                          **{"project_id": self.project_id})

    def test_error_thrown_when_no_session_and_no_project_id(self):
        self.assertRaises(ValueError, client.Client,
                          **{"endpoint": self.endpoint})

    def test_endpoint_override_starts_with_endpoint_url(self):
        c = client.Client(session=self.session,
                          endpoint=self.endpoint,
                          project_id=self.project_id)
        self.assertTrue(c.client.endpoint_override.startswith(self.endpoint))

    def test_endpoint_override_ends_with_default_api_version(self):
        c = client.Client(session=self.session,
                          endpoint=self.endpoint,
                          project_id=self.project_id)
        self.assertTrue(c.client.endpoint_override.rstrip('/').endswith(
            client._DEFAULT_API_VERSION
        ))


class WhenTestingClientPost(TestClient):

    def setUp(self):
        super(WhenTestingClientPost, self).setUp()
        self.httpclient = client._HTTPClient(
            session=self.session,
            microversion=_DEFAULT_MICROVERSION,
            endpoint=self.endpoint,
            version='v1')
        self.href = self.endpoint + '/v1/secrets/'
        self.post_mock = self.responses.post(self.href, json={})

    def test_post_normalizes_url_with_traling_slash(self):
        self.httpclient.post(path='secrets', json={'test_data': 'test'})
        self.assertTrue(self.post_mock.last_request.url.endswith('/'))

    def test_post_includes_content_type_header_of_application_json(self):
        self.httpclient.post(path='secrets', json={'test_data': 'test'})
        self.assertEqual('application/json',
                         self.post_mock.last_request.headers['Content-Type'])

    def test_post_includes_default_headers(self):
        self.httpclient._default_headers = {'Test-Default-Header': 'test'}
        self.httpclient.post(path='secrets', json={'test_data': 'test'})
        self.assertEqual(
            'test',
            self.post_mock.last_request.headers['Test-Default-Header'])

    def test_post_checks_status_code(self):
        self.httpclient._check_status_code = mock.MagicMock()
        self.httpclient.post(path='secrets', json={'test_data': 'test'})
        self.httpclient._check_status_code.assert_has_calls([])


class WhenTestingClientPut(TestClient):

    def setUp(self):
        super(WhenTestingClientPut, self).setUp()
        self.httpclient = client._HTTPClient(
            session=self.session,
            microversion=_DEFAULT_MICROVERSION,
            endpoint=self.endpoint)
        self.href = 'http://test_href/'
        self.put_mock = self.responses.put(self.href, status_code=204)

    def test_put_uses_href_as_is(self):
        self.httpclient.put(self.href)
        self.assertTrue(self.put_mock.called)

    def test_put_passes_data(self):
        data = "test"
        self.httpclient.put(self.href, data=data)
        self.assertEqual("test", self.put_mock.last_request.text)

    def test_put_includes_default_headers(self):
        self.httpclient._default_headers = {'Test-Default-Header': 'test'}
        self.httpclient.put(self.href)
        self.assertEqual(
            'test',
            self.put_mock.last_request.headers['Test-Default-Header'])

    def test_put_checks_status_code(self):
        self.httpclient._check_status_code = mock.MagicMock()
        self.httpclient.put(self.href, data='test')
        self.httpclient._check_status_code.assert_has_calls([])


class WhenTestingClientGet(TestClient):

    def setUp(self):
        super(WhenTestingClientGet, self).setUp()
        self.httpclient = client._HTTPClient(
            session=self.session,
            microversion=_DEFAULT_MICROVERSION,
            endpoint=self.endpoint)
        self.headers = dict()
        self.href = 'http://test_href/'
        self.get_mock = self.responses.get(self.href, json={})

    def test_get_uses_href_as_is(self):
        self.httpclient.get(self.href)
        self.assertEqual(self.href, self.get_mock.last_request.url)

    def test_get_passes_params(self):
        params = {'test': 'test1'}
        self.httpclient.get(self.href, params=params)
        self.assertEqual(self.href,
                         self.get_mock.last_request.url.split('?')[0])
        self.assertEqual(['test1'], self.get_mock.last_request.qs['test'])

    def test_get_includes_accept_header_of_application_json(self):
        self.httpclient.get(self.href)
        self.assertEqual('application/json',
                         self.get_mock.last_request.headers['Accept'])

    def test_get_includes_default_headers(self):
        self.httpclient._default_headers = {'Test-Default-Header': 'test'}
        self.httpclient.get(self.href)
        self.assertEqual(
            'test',
            self.get_mock.last_request.headers['Test-Default-Header'])

    def test_get_checks_status_code(self):
        self.httpclient._check_status_code = mock.MagicMock()
        self.httpclient.get(self.href)
        self.httpclient._check_status_code.assert_has_calls([])

    def test_get_raw_uses_href_as_is(self):
        self.httpclient._get_raw(self.href, headers=self.headers)
        self.assertEqual(self.href, self.get_mock.last_request.url)

    def test_get_raw_passes_headers(self):
        self.httpclient._get_raw(self.href, headers={'test': 'test'})
        self.assertEqual('test', self.get_mock.last_request.headers['test'])

    def test_get_raw_includes_default_headers(self):
        self.httpclient._default_headers = {'Test-Default-Header': 'test'}
        self.httpclient._get_raw(self.href, headers=self.headers)
        self.assertIn('Test-Default-Header',
                      self.get_mock.last_request.headers)

    def test_get_raw_checks_status_code(self):
        self.httpclient._check_status_code = mock.MagicMock()
        self.httpclient._get_raw(self.href, headers=self.headers)
        self.httpclient._check_status_code.assert_has_calls([])


class WhenTestingClientDelete(TestClient):

    def setUp(self):
        super(WhenTestingClientDelete, self).setUp()
        self.httpclient = client._HTTPClient(
            session=self.session,
            microversion=_DEFAULT_MICROVERSION,
            endpoint=self.endpoint)
        self.href = 'http://test_href/'
        self.del_mock = self.responses.delete(self.href, status_code=204)

    def test_delete_uses_href_as_is(self):
        self.httpclient.delete(self.href)
        self.assertTrue(self.del_mock.called)

    def test_delete_passes_json(self):
        json = {"test": "test"}
        self.httpclient.delete(self.href, json=json)
        self.assertEqual('{"test": "test"}', self.del_mock.last_request.text)

    def test_delete_includes_default_headers(self):
        self.httpclient._default_headers = {'Test-Default-Header': 'test'}
        self.httpclient.delete(self.href)
        self.assertEqual(
            'test',
            self.del_mock.last_request.headers['Test-Default-Header'])

    def test_delete_checks_status_code(self):
        self.httpclient._check_status_code = mock.MagicMock()
        self.httpclient.delete(self.href)
        self.httpclient._check_status_code.assert_has_calls([])


class WhenTestingCheckStatusCodes(TestClient):

    def test_raises_http_auth_error_for_401_response(self):
        resp = mock.MagicMock()
        resp.status_code = 401
        self.assertRaises(exceptions.HTTPAuthError,
                          self.httpclient._check_status_code,
                          resp)

    def test_raises_http_server_error_for_500_response(self):
        resp = mock.MagicMock()
        resp.status_code = 500
        self.assertRaises(exceptions.HTTPServerError,
                          self.httpclient._check_status_code, resp)

    def test_raises_http_client_error_for_400_response(self):
        resp = mock.MagicMock()
        resp.status_code = 400
        self.assertRaises(exceptions.HTTPClientError,
                          self.httpclient._check_status_code, resp)


class WhenTestingGetErrorMessage(TestClient):

    def test_gets_error_message_from_title_in_json(self):
        resp = mock.MagicMock()
        resp.json.return_value = {'title': 'test_text'}
        msg = self.httpclient._get_error_message(resp)
        self.assertEqual('test_text', msg)

    def test_gets_error_message_from_content_when_no_json(self):
        resp = mock.MagicMock()
        resp.json.side_effect = ValueError()
        resp.content = content = 'content'
        msg = self.httpclient._get_error_message(resp)
        self.assertEqual(content, msg)

    def test_gets_error_message_from_description_in_json(self):
        resp = mock.MagicMock()
        resp.json.return_value = {'title': 'test_text',
                                  'description': 'oopsie'}
        msg = self.httpclient._get_error_message(resp)
        self.assertEqual('test_text: oopsie', msg)


class BaseEntityResource(TestClient):

    def _setUp(self, entity, entity_id='abcd1234-eabc-5678-9abc-abcdef012345'):
        # TODO(dmendiza) Why are we calling super().setUp() from _setUp()?
        super(BaseEntityResource, self).setUp()

        self.entity = entity
        self.entity_id = entity_id
        self.entity_base = self.endpoint + "/v1/" + self.entity
        self.entity_href = self.entity_base + "/" + self.entity_id
        self.entity_payload_href = self.entity_href + "/payload"

        self.client = client.Client(endpoint=self.endpoint,
                                    project_id=self.project_id)


class WhenTestingClientMicroversion(TestClient):
    def _create_mock_session(
            self, requested_version, server_max_version, server_min_version,
            endpoint):
        sess = mock_session()

        mock_session_get_endpoint(sess, get_version_endpoint(endpoint))
        mock_session_get(
            sess, get_server_supported_versions(
                server_min_version, server_max_version))

        return sess

    def _test_client_creation_with_endpoint(
            self, requested_version, server_max_version, server_min_version,
            endpoint):
        sess = self._create_mock_session(
            requested_version, server_max_version, server_min_version,
            endpoint)

        client.Client(session=sess, microversion=requested_version)

        headers = {
            'Accept': 'application/json',
            'OpenStack-API-Version': 'key-manager 1.1'
        }

        sess.get.assert_called_with(
            get_version_endpoint(endpoint), headers=headers,
            authenticated=None)

    def _mock_session_and_get_client(
            self, requested_version, server_max_version, server_min_version,
            endpoint=None):
        sess = self._create_mock_session(
            requested_version, server_max_version, server_min_version,
            endpoint)

        return client.Client(session=sess, microversion=requested_version)

    def test_fails_when_requesting_invalid_microversion(self):
        self.assertRaises(TypeError,
                          client.Client, session=self.session,
                          endpoint=self.endpoint, project_id=self.project_id,
                          microversion="a")

    def test_fails_when_requesting_unsupported_microversion(self):
        self.assertRaises(UnsupportedVersion,
                          client.Client, session=self.session,
                          endpoint=self.endpoint, project_id=self.project_id,
                          microversion="1.9")

    def test_fails_when_requesting_unsupported_version(self):
        self.assertRaises(UnsupportedVersion,
                          client.Client, session=self.session,
                          endpoint=self.endpoint, project_id=self.project_id,
                          version="v0")

    def test_passes_without_providing_endpoint(self):
        requested_version = None
        server_max_version = (1, 1)
        server_min_version = (1, 0)
        endpoint = None

        self._test_client_creation_with_endpoint(
            requested_version, server_max_version, server_min_version,
            endpoint)

    def test_passes_with_custom_endpoint(self):
        requested_version = None
        server_max_version = (1, 1)
        server_min_version = (1, 0)
        endpoint = self.endpoint

        self._test_client_creation_with_endpoint(
            requested_version, server_max_version, server_min_version,
            endpoint)

    def test_passes_with_default_microversion_as_1_1(self):
        requested_version = None
        server_max_version = (1, 1)
        server_min_version = (1, 0)

        c = self._mock_session_and_get_client(
            requested_version, server_max_version, server_min_version)

        self.assertEqual("1.1", c.client.microversion)

    def test_passes_with_default_microversion_as_1_0(self):
        requested_version = None
        server_max_version = (1, 0)
        server_min_version = (1, 0)

        c = self._mock_session_and_get_client(
            requested_version, server_max_version, server_min_version)

        self.assertEqual("1.0", c.client.microversion)

    def test_fails_requesting_higher_microversion_than_supported_by_server(
            self):
        requested_version = "1.1"
        server_max_version = (1, 0)
        server_min_version = (1, 0)

        sess = self._create_mock_session(
            requested_version, server_max_version, server_min_version,
            self.endpoint)

        self.assertRaises(
            UnsupportedVersion, client.Client, session=sess,
            endpoint=self.endpoint, microversion=requested_version)

    def test_fails_requesting_lower_microversion_than_supported_by_server(
            self):
        requested_version = "1.0"
        server_max_version = (1, 1)
        server_min_version = (1, 1)

        sess = self._create_mock_session(
            requested_version, server_max_version, server_min_version,
            self.endpoint)

        self.assertRaises(
            UnsupportedVersion, client.Client, session=sess,
            endpoint=self.endpoint, microversion=requested_version)

    def test_passes_with_stable_server_version(self):
        requested_version = "1.0"
        server_max_version = None
        server_min_version = None

        c = self._mock_session_and_get_client(
            requested_version, server_max_version, server_min_version)

        self.assertEqual(requested_version, c.client.microversion)



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import datetime
import functools
from os import path
import time
import types

import oslotest.base as oslotest
import urllib.parse as urlparse


class BaseTestCase(oslotest.BaseTestCase):
    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.order_id = 'order1234'
        self.external_project_id = 'keystone1234'

    def tearDown(self):
        super(BaseTestCase, self).tearDown()


def construct_new_test_function(original_func, name, build_params):
    """Builds a new test function based on parameterized data.

    :param original_func: The original test function that is used as a template
    :param name: The fullname of the new test function
    :param build_params: A dictionary or list containing args or kwargs
        for the new test
    :return: A new function object
    """
    new_func = types.FunctionType(
        original_func.__code__,
        original_func.__globals__,
        name=name,
        argdefs=original_func.__defaults__
    )

    # Support either an arg list or kwarg dict for our data
    build_args = build_params if isinstance(build_params, list) else []
    build_kwargs = build_params if isinstance(build_params, dict) else {}

    # Build a test wrapper to execute with our kwargs
    def test_wrapper(func, test_args, test_kwargs):
        @functools.wraps(func)
        def wrapper(self):
            return func(self, *test_args, **test_kwargs)
        return wrapper

    return test_wrapper(new_func, build_args, build_kwargs)


def process_parameterized_function(name, func_obj, build_data):
    """Build lists of functions to add and remove to a test case."""
    to_remove = []
    to_add = []

    for subtest_name, params in build_data.items():
        # Build new test function
        func_name = '{0}_{1}'.format(name, subtest_name)
        new_func = construct_new_test_function(func_obj, func_name, params)

        # Mark the new function as needed to be added to the class
        to_add.append((func_name, new_func))

        # Mark key for removal
        to_remove.append(name)

    return to_remove, to_add


def parameterized_test_case(cls):
    """Class decorator to process parameterized tests

    This allows for parameterization to be used for potentially any
    unittest compatible runner; including testr and py.test.
    """
    tests_to_remove = []
    tests_to_add = []
    for key, val in vars(cls).items():
        # Only process tests with build data on them
        if key.startswith('test_') and val.__dict__.get('build_data'):
            to_remove, to_add = process_parameterized_function(
                name=key,
                func_obj=val,
                build_data=val.__dict__.get('build_data')
            )
            tests_to_remove.extend(to_remove)
            tests_to_add.extend(to_add)

    # Add all new test functions
    [setattr(cls, name, func) for name, func in tests_to_add]

    # Remove all old test function templates (if they still exist)
    [delattr(cls, key) for key in tests_to_remove if hasattr(cls, key)]
    return cls


def parameterized_dataset(build_data):
    """Simple decorator to mark a test method for processing."""
    def decorator(func):
        func.__dict__['build_data'] = build_data
        return func
    return decorator


def create_timestamp_w_tz_and_offset(timezone=None, days=0, hours=0, minutes=0,
                                     seconds=0):
    """Creates a timestamp with a timezone and offset in days

    :param timezone: Timezone used in creation of timestamp
    :param days: The offset in days
    :param hours: The offset in hours
    :param minutes: The offset in minutes

    :return: a timestamp
    """
    if timezone is None:
        timezone = time.strftime("%z")

    timestamp = '{time}{timezone}'.format(
        time=(datetime.datetime.today() + datetime.timedelta(days=days,
                                                             hours=hours,
                                                             minutes=minutes,
                                                             seconds=seconds)),
        timezone=timezone)

    return timestamp


def get_limit_and_offset_from_ref(ref):
    matches = dict(urlparse.parse_qsl(urlparse.urlparse(ref).query))
    ref_limit = matches['limit']
    ref_offset = matches['offset']

    return ref_limit, ref_offset


def get_tomorrow_timestamp():
    tomorrow = (datetime.today() + datetime.timedelta(days=1))
    return tomorrow.isoformat()


def string_to_datetime(datetimestring, date_formats=None):
    date_formats = date_formats or [
        '%Y-%m-%d %H:%M:%S', '%Y-%m-%d %H:%M:%S.%f', '%Y-%m-%dT%H:%M:%S.%fZ',
        '%Y-%m-%dT%H:%M:%S.%f', "%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%dT%H:%M:%S"]

    for dateformat in date_formats:
        try:
            return datetime.datetime.strptime(datetimestring, dateformat)
        except ValueError:
            continue
    else:
        raise


def get_id_from_ref(ref):
    """Returns id from reference."""
    ref_id = None
    if ref is not None and len(ref) > 0:
        ref_id = path.split(ref)[1]
    return ref_id



# Copyright (c) 2015 Ericsson AB.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from barbicanclient import client
from barbicanclient import exceptions
from functionaltests.base import BaseTestCase
from functionaltests.common import config
from keystoneauth1 import exceptions as ks_exceptions
from keystoneauth1 import identity
from keystoneauth1 import session

CONF = config.get_config()


class WhenTestingClientConnectivity(BaseTestCase):

    @classmethod
    def setUpClass(cls):
        super(WhenTestingClientConnectivity, cls).setUpClass()
        if 'v2' in CONF.identity.auth_version:
            cls.auth = identity.Password(
                auth_url=CONF.identity.uri,
                username=CONF.keymanager.username,
                password=CONF.keymanager.password,
                tenant_name=CONF.keymanager.project_name)
        else:
            cls.auth = identity.Password(
                auth_url=CONF.identity.uri,
                username=CONF.keymanager.username,
                user_domain_name=CONF.identity.domain_name,
                password=CONF.keymanager.password,
                project_name=CONF.keymanager.project_name,
                project_domain_name=CONF.keymanager.project_domain_name)

        # enables the tests in this class to share a keystone token
        cls.sess = session.Session(auth=cls.auth)

    def setUp(self):
        self.LOG.info('Starting: %s', self._testMethodName)
        super(WhenTestingClientConnectivity, self).setUp()

    def assert_client_can_contact_barbican(self, client):
        """Asserts that the client has connectivity to Barbican.

        If there was an error with the connectivity, the operations that are
        attempted through the client would throw an exception.
        """
        containers = client.containers.list()
        orders = client.orders.list()
        secrets = client.secrets.list()

        self.assertIsNotNone(containers)
        self.assertIsNotNone(orders)
        self.assertIsNotNone(secrets)

    def assert_client_cannot_contact_barbican(self, client):
        self.assertRaises(exceptions.HTTPClientError, client.containers.list)
        self.assertRaises(exceptions.HTTPClientError, client.orders.list)
        self.assertRaises(exceptions.HTTPClientError, client.secrets.list)

    def test_can_access_server_if_endpoint_and_session_specified(self):
        barbicanclient = client.Client(
            endpoint=CONF.keymanager.url,
            project_id=CONF.keymanager.project_id,
            session=self.sess)

        self.assert_client_can_contact_barbican(barbicanclient)

    def test_client_can_access_server_if_no_endpoint_specified(self):
        barbicanclient = client.Client(
            project_id=CONF.keymanager.project_id,
            session=self.sess)

        self.assert_client_can_contact_barbican(barbicanclient)

    def test_client_can_access_server_if_no_session_specified(self):
        barbicanclient = client.Client(
            endpoint=CONF.keymanager.url,
            project_id=CONF.keymanager.project_id,
            auth=self.auth)

        self.assert_client_can_contact_barbican(barbicanclient)

    def test_client_can_access_server_if_endpoint_filters_specified(self):
        barbicanclient = client.Client(
            project_id=CONF.keymanager.project_id,
            auth=self.auth,
            interface=client._DEFAULT_SERVICE_INTERFACE,
            service_type=client._DEFAULT_SERVICE_TYPE,
            version=client._DEFAULT_API_VERSION)

        self.assert_client_can_contact_barbican(barbicanclient)

    def test_client_cannot_access_server_if_endpoint_filter_wrong(self):
        self.assertRaises(
            ks_exceptions.EndpointNotFound,
            client.Client,
            project_id=CONF.keymanager.project_id,
            auth=self.auth,
            interface=client._DEFAULT_SERVICE_INTERFACE,
            service_type='wrong-service-type',
            version=client._DEFAULT_API_VERSION)

        self.assertRaises(
            ks_exceptions.EndpointNotFound,
            client.Client,
            project_id=CONF.keymanager.project_id,
            auth=self.auth,
            interface='wrong-interface',
            service_type=client._DEFAULT_SERVICE_TYPE,
            version=client._DEFAULT_API_VERSION)

        self.assertRaises(
            ks_exceptions.EndpointNotFound,
            client.Client,
            project_id=CONF.keymanager.project_id,
            auth=self.auth,
            interface=client._DEFAULT_SERVICE_INTERFACE,
            service_type=client._DEFAULT_SERVICE_TYPE,
            service_name='wrong-service-name',
            version=client._DEFAULT_API_VERSION)

        self.assertRaises(
            ks_exceptions.EndpointNotFound,
            client.Client,
            project_id=CONF.keymanager.project_id,
            auth=self.auth,
            interface=client._DEFAULT_SERVICE_INTERFACE,
            service_type=client._DEFAULT_SERVICE_TYPE,
            region_name='wrong-region-name',
            version=client._DEFAULT_API_VERSION)

    def test_cannot_create_client_if_nonexistent_version_specified(self):
        self.assertRaises(exceptions.UnsupportedVersion,
                          client.Client,
                          **{"project_id": CONF.keymanager.project_id,
                             "auth": self.auth,
                             "interface": client._DEFAULT_SERVICE_INTERFACE,
                             "service_type": client._DEFAULT_SERVICE_TYPE,
                             "version": 'wrong-version'})

        self.assertRaises(exceptions.UnsupportedVersion,
                          client.Client,
                          **{"endpoint": CONF.keymanager.url,
                             "project_id": CONF.keymanager.project_id,
                             "auth": self.auth,
                             "version": 'nonexistent_version'})

    def test_client_can_access_server_if_no_version_is_specified(self):
        barbicanclient = client.Client(
            project_id=CONF.keymanager.project_id,
            auth=self.auth,
            interface=client._DEFAULT_SERVICE_INTERFACE,
            service_type=client._DEFAULT_SERVICE_TYPE)

        self.assert_client_can_contact_barbican(barbicanclient)
